from PyQt5 import QtGui, QtWidgets
from PyQt5.QtWidgets import QApplication

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui
from oasys.util.oasys_util import EmittingStream

from orangecontrib.shadow.util.shadow_objects import ShadowBeam
from orangecontrib.shadow.util.shadow_util import ShadowCongruence, ShadowPlot
from orangecontrib.shadow.widgets.gui.ow_automatic_element import AutomaticElement
from orangecontrib.esrf.shadow.widgets.gui.plots import plot_data1D, plot_data2D
from srxraylib.util.h5_simple_writer import H5SimpleWriter

import numpy
import sys


class ESRFCaustic(AutomaticElement):

    name = "Caustic Generator"
    description = "Shadow: Caustic Generator"
    icon = "icons/caustic.png"
    maintainer = "APS+ESRF team"
    maintainer_email = "srio@esrf.eu, srio@lbl.gov"
    priority = 5
    category = "Data Display Tools"
    keywords = ["data", "file", "load", "read"]

    inputs = [("Input Beam", ShadowBeam, "setBeam")]

    MAX_WIDTH = 1320
    MAX_HEIGHT = 700

    IMAGE_WIDTH = 860
    IMAGE_HEIGHT = 645
    #
    # CONTROL_AREA_WIDTH = 405
    # TABS_AREA_HEIGHT = 650



    want_main_area=1
    want_control_area = 1

    input_beam=None

    npoints_x = Setting(1000) # in X and Z
    npoints_z = Setting(101)  # in X and Z
    npositions = Setting(300) # in Y



    y_min=Setting(-5.0)
    y_max=Setting(5.0)

    no_lost  = Setting(1)
    use_reflectivity = Setting(1)
    shadow_column = Setting(0)

    x_min = Setting(-0.2)
    x_max = Setting( 0.2)

    save_h5_file_flag = Setting(0)
    save_h5_file_name = Setting("caustic.h5")

    def __init__(self, show_automatic_box=True):
        super().__init__()

        try:
            tmp = self.workspace_units_label
        except:
            self.workspace_units = 0
            self.workspace_units_label = "m"
            self.workspace_units_to_m = 1.0
            self.workspace_units_to_cm = 100.0
            self.workspace_units_to_mm = 1000.0

        gui.button(self.controlArea, self, "Calculate", callback=self.calculate, height=45)

        general_box = oasysgui.widgetBox(self.controlArea, "General Settings", addSpace=True, orientation="vertical",)
                                         # width=self.CONTROL_AREA_WIDTH-8, height=400)


        gui.comboBox(general_box, self, "no_lost", label="Rays",labelWidth=220,
                                     items=["All rays","Good only","Lost only"],
                                     sendSelectedValue=False, orientation="horizontal")


        gui.comboBox(general_box, self, "use_reflectivity", label="Include reflectivity",labelWidth=220,
                                     items=["No","Yes"],
                                     sendSelectedValue=False, orientation="horizontal")

        box_y = oasysgui.widgetBox(general_box, "Propagation along Y (col 2)", addSpace=True, orientation="vertical", height=100)
        oasysgui.lineEdit(box_y, self, "npositions", "Y Points", labelWidth=260, valueType=int, orientation="horizontal")
        oasysgui.lineEdit(box_y, self, "y_min", "Y min"+ " [" + self.workspace_units_label + "]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(box_y, self, "y_max", "Y max"+ " [" + self.workspace_units_label + "]", labelWidth=260, valueType=float, orientation="horizontal")


        gui.comboBox(general_box, self, "shadow_column", label="Scan direction",labelWidth=220,
                                     items=["X (col 1)","Z (col 3)", "R (col 20)"],
                                     sendSelectedValue=False, orientation="horizontal")

        box_x = oasysgui.widgetBox(general_box, "Scan direction", addSpace=True, orientation="vertical", height=100)
        oasysgui.lineEdit(box_x, self, "npoints_x", "Points", labelWidth=260, valueType=int,orientation="horizontal")
        oasysgui.lineEdit(box_x, self, "x_min", "min"+ " [" + self.workspace_units_label + "]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(box_x, self, "x_max", "max"+ " [" + self.workspace_units_label + "]", labelWidth=260, valueType=float, orientation="horizontal")

        gui.separator(self.controlArea, height=200)

        box_file = oasysgui.widgetBox(general_box, "File", addSpace=True, orientation="vertical", height=100)
        gui.comboBox(box_file, self, "save_h5_file_flag", label="Save plots into h5 file", labelWidth=250,
                                         items=["No", "Yes"],
                                         sendSelectedValue=False, orientation="horizontal",callback=self.set_visible)

        self.box_file_1 = oasysgui.widgetBox(box_file, "", addSpace=False, orientation="horizontal", height=25)
        oasysgui.lineEdit(self.box_file_1, self, "save_h5_file_name", "File Name", labelWidth=100,  valueType=str, orientation="horizontal")



        #
        #
        #
        tabs_setting = oasysgui.tabWidget(self.mainArea)
        tabs_setting.setFixedHeight(self.IMAGE_HEIGHT+5)
        tabs_setting.setFixedWidth(self.IMAGE_WIDTH)


        tmp = oasysgui.createTabPage(tabs_setting, "Intensity vs y")
        self.image_box = gui.widgetBox(tmp, "", addSpace=True, orientation="vertical")
        self.image_box.setFixedHeight(self.IMAGE_HEIGHT-30)
        self.image_box.setFixedWidth(self.IMAGE_WIDTH-20)

        tmp = oasysgui.createTabPage(tabs_setting, "FWHM(y)")
        self.box_fwhm = gui.widgetBox(tmp, "", addSpace=True, orientation="vertical")
        self.box_fwhm.setFixedHeight(self.IMAGE_HEIGHT-30)
        self.box_fwhm.setFixedWidth(self.IMAGE_WIDTH-20)

        tmp = oasysgui.createTabPage(tabs_setting, "Center(y)")
        self.box_center = gui.widgetBox(tmp, "", addSpace=True, orientation="vertical")
        self.box_center.setFixedHeight(self.IMAGE_HEIGHT-30)
        self.box_center.setFixedWidth(self.IMAGE_WIDTH-20)

        tmp = oasysgui.createTabPage(tabs_setting, "I0(y)")
        self.box_I0 = gui.widgetBox(tmp, "", addSpace=True, orientation="vertical")
        self.box_I0.setFixedHeight(self.IMAGE_HEIGHT-30)
        self.box_I0.setFixedWidth(self.IMAGE_WIDTH-20)

        tmp = oasysgui.createTabPage(tabs_setting, "Info")
        self.focnewInfo = oasysgui.textArea(height=self.IMAGE_HEIGHT-35)
        info_box = oasysgui.widgetBox(tmp, "", addSpace=True, orientation="horizontal", height = self.IMAGE_HEIGHT-20, width = self.IMAGE_WIDTH-20)
        info_box.layout().addWidget(self.focnewInfo)


        self.set_visible()

    def set_visible(self):
        self.box_file_1.setVisible(self.save_h5_file_flag != 0)

    def writeStdOut(self, text="", initialize=False):
        cursor = self.focnewInfo.textCursor()
        if initialize:
            self.focnewInfo.setText(text)
        else:
            cursor.movePosition(QtGui.QTextCursor.End)
            cursor.insertText(text)

    def setBeam(self, beam):
        if ShadowCongruence.checkEmptyBeam(beam):
            if ShadowCongruence.checkGoodBeam(beam):
                self.input_beam = beam

                if self.is_automatic_run:
                    self.calculate()
            else:
                QtWidgets.QMessageBox.critical(self, "Error",
                                           "Data not displayable: No good rays or bad content",
                                           QtWidgets.QMessageBox.Ok)
    def get_shadow3_beam(self):
        if ShadowCongruence.checkEmptyBeam(self.input_beam):
            if ShadowCongruence.checkGoodBeam(self.input_beam):

                beam_to_analize = self.input_beam._beam

                return beam_to_analize


    def calculate(self):

        self.progressBarInit()

        # capture stout
        self.writeStdOut(initialize=True)
        sys.stdout = EmittingStream(textWritten=self.writeStdOut)

        beam_to_analize = self.get_shadow3_beam()

        if beam_to_analize is None:
            print("No SHADOW Beam")
            return

        if beam_to_analize is None:
            return

        positions = numpy.linspace(self.y_min, self.y_max, self.npositions)

        out_x = numpy.zeros((self.npoints_x, self.npositions))
        fwhm = numpy.zeros(self.npositions)
        center = numpy.zeros(self.npositions)

        if self.shadow_column == 0:
            col = 1
        elif self.shadow_column == 1:
            col = 3
        elif self.shadow_column == 2:
            col = 20

        if self.use_reflectivity:
            ref = 23
        else:
            ref = 0

        self.progressBarSet(10)
        self.setStatusMessage("Retracing...")
        for i in range(self.npositions):
            if numpy.mod(i,10) == 0:
                print("Calculating position %d of %d"%(i+1,self.npositions))
                self.progressBarSet(10 + 85 * i / self.npositions)
            beami = beam_to_analize.duplicate()
            beami.retrace(positions[i], resetY=True)
            tkt_x = beami.histo1(col, xrange=[self.x_min, self.x_max], nbins=self.npoints_x, nolost=self.no_lost, ref=ref)
            out_x[:, i] = tkt_x["histogram"]
            fwhm[i] = tkt_x["fwhm"]

            if ref == 23:
                center[i] = numpy.average(beami.getshonecol(col, nolost=self.no_lost),
                                          weights=beami.getshonecol(23, nolost=self.no_lost))
            else:
                center[i] = numpy.average(beami.getshonecol(col,nolost=self.no_lost),)


        #
        # plots
        #
        print("\nResult arrays X,Y (shapes): ", out_x.shape, tkt_x["bin_center"].shape, positions.shape )
        x = tkt_x["bin_center"]
        y = positions

        if self.shadow_column == 0:
            col_title="X (col 1)"
        elif self.shadow_column == 1:
            col_title = "Z (col 3)"
        elif self.shadow_column == 2:
            col_title = "R (col 20)"

        plot_canvas = plot_data2D(
                             out_x.T, y, 1e6 * self.workspace_units_to_m * x,
                             title="",ytitle="%s [um] (%d pixels)"%(col_title,x.size),xtitle="Y [%s] (%d pixels)"%(self.workspace_units_label,y.size),)
        self.image_box.layout().removeItem(self.image_box.layout().itemAt(0))
        self.image_box.layout().addWidget(plot_canvas)


        #FWHM
        fwhm[fwhm == 0] = 'nan'
        self.box_fwhm.layout().removeItem(self.box_fwhm.layout().itemAt(0))
        plot_widget_id = plot_data1D(y,1e6 * self.workspace_units_to_m * fwhm,title="FWHM",xtitle="y [%s]"%self.workspace_units_label,ytitle="FHWH [um]",symbol='.')
        self.box_fwhm.layout().addWidget(plot_widget_id)

        #I0
        nx, ny = out_x.shape
        I0 = out_x.T[:,nx//2]
        self.box_I0.layout().removeItem(self.box_I0.layout().itemAt(0))
        plot_widget_id = plot_data1D(y,I0,title="I at central profile",xtitle="y [%s]"%self.workspace_units_label,ytitle="I0",symbol='.')
        self.box_I0.layout().addWidget(plot_widget_id)

        #center
        self.box_center.layout().removeItem(self.box_center.layout().itemAt(0))
        plot_widget_id = plot_data1D(y, 1e6 * self.workspace_units_to_m * center,title="CENTER",xtitle="y [%s]"%self.workspace_units_label,ytitle="CENTER [um]",symbol='.',
                                     yrange=[1e6 * self.workspace_units_to_m * self.x_min, 1e6 * self.workspace_units_to_m * self.x_max])
        self.box_center.layout().addWidget(plot_widget_id)



        if self.save_h5_file_flag:

            h5w = H5SimpleWriter.initialize_file(self.save_h5_file_name, creator="h5_basic_writer.py")

            h5w.create_entry("caustic", nx_default="image")

            h5w.add_image(out_x.T, y, 1e6 * self.workspace_units_to_m * x,
                          entry_name="caustic", image_name="image",
                          title_y="%s [um] (%d pixels)"%(col_title,x.size),
                          title_x="Y [%s] (%d pixels)"%(self.workspace_units_label, y.size),)

            h5w.add_dataset(y, 1e6 * self.workspace_units_to_m * fwhm,
                            entry_name="caustic", dataset_name="fwhm",
                            title_x="Y [%s]" % self.workspace_units_label, title_y="FWHM [um]")

            h5w.add_dataset(y, 1e6 * self.workspace_units_to_m * center,
                            entry_name="caustic", dataset_name="center",
                            title_x="Y [%s]" % self.workspace_units_label, title_y="center [um]")

            h5w.add_dataset(y, I0,
                            entry_name="caustic", dataset_name="I0",
                            title_x="Y [%s]" % self.workspace_units_label, title_y="I at central profile")

        self.progressBarFinished()


if __name__ == "__main__":
    import sys
    import Shadow

    class MyBeam():
        pass
    beam_to_analize = Shadow.Beam()
    beam_to_analize.load("/Users/srio/Oasys/lens.01")
    my_beam = MyBeam()
    my_beam._beam = beam_to_analize

    a = QApplication(sys.argv)
    ow = ESRFCaustic()


    ow.show()

    ow.npoints_x = 30
    ow.npoints_z = 30
    ow.npositions = 50
    ow.y_min =0
    ow.y_max = 100
    ow.shadow_column = 1

    ow.x_min = -350e-6
    ow.x_max = 350e-6
    ow.save_h5_file_flag = 1


    ow.input_beam = my_beam
    a.exec_()
    ow.saveSettings()



import sys, os, platform
from PyQt5.QtWidgets import QApplication

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui, congruence

from orangecontrib.xoppy.util.xoppy_util import locations
from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget

import numpy
import scipy.constants as codata

from syned.widget.widget_decorator import WidgetDecorator
import syned.beamline.beamline as synedb
import syned.storage_ring.magnetic_structures.insertion_device as synedid

class OWID19ws(XoppyWidget,WidgetDecorator):
    name = "ID19WS"
    id = "orange.widgets.dataws"
    description = "Wiggler Spectrum on a Screen"
    icon = "icons/ws_id19_hpw150.png"
    priority = 1
    category = ""
    keywords = ["xoppy", "ws"]

    ENERGY = Setting(6.0)
    CUR = Setting(200.0)
    PERIOD = Setting(15.0)
    N = Setting(28)
    KX = Setting(0.0)
    KY = Setting(19)
    GAP = Setting(30)
    EMIN = Setting(1000.0)
    EMAX = Setting(200000.0)
    NEE = Setting(500)
    D = Setting(30.0)
    XPC = Setting(0.0)
    YPC = Setting(0.0)
    XPS = Setting(2.0)
    YPS = Setting(2.0)
    NXP = Setting(10)
    NYP = Setting(10)

    inputs = WidgetDecorator.syned_input_data()

    def build_gui(self):

        box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)

        idx = -1 


        #widget index 1
        idx += 1 
        box1 = gui.widgetBox(box) 
        self.id_ENERGY = oasysgui.lineEdit(box1, self, "ENERGY",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 2 
        idx += 1 
        box1 = gui.widgetBox(box) 
        self.id_CUR = oasysgui.lineEdit(box1, self, "CUR",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 3 
        idx += 1 
        box1 = gui.widgetBox(box) 
        self.id_PERIOD = oasysgui.lineEdit(box1, self, "PERIOD",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 4 
        idx += 1 
        box1 = gui.widgetBox(box) 
        self.id_N = oasysgui.lineEdit(box1, self, "N",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 

        # COMMENTED AS IT IS NOT IMPLEMENTED!!
        #widget index 5 
        # idx += 1
        # box1 = gui.widgetBox(box)
        # oasysgui.lineEdit(box1, self, "KX",
        #              label=self.unitLabels()[idx], addSpace=False,
        #             valueType=float, orientation="horizontal", labelWidth=250)
        # self.show_at(self.unitFlags()[idx], box1)
        
        #widget index 6 
        idx += 1 
        box1 = gui.widgetBox(box) 
        self.id_KY = oasysgui.lineEdit(box1, self, "KY",
                     valueType=float, label=self.unitLabels()[idx], addSpace=False, orientation="horizontal", labelWidth=250, callback=self.calculateGAP)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 7
        idx += 1
        box1 = gui.widgetBox(box)
        self.id_GAP = oasysgui.lineEdit(box1, self, "GAP",
                     valueType=float, label=self.unitLabels()[idx], addSpace=False, orientation="horizontal", labelWidth=250, callback=self.calculateKY)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 8
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "EMIN",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 9
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "EMAX",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 10
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "NEE",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 11
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "D",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 12
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "XPC",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 13
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "YPC",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 14
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "XPS",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 15
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "YPS",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 16
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "NXP",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 17
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "NYP",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

    def calculateGAP(self):
        self.GAP=numpy.round((-1 / 0.02677) * numpy.log(self.KY / (93.4 * 2.89408 * self.PERIOD * 0.01)), 5)

    def calculateKY(self):
        self.KY=numpy.round((93.4 * 0.01 * self.PERIOD * (2.3333 * numpy.exp(-0.02473 * self.GAP) + 1.189 * numpy.exp(-0.059691 * self.GAP))), 5)


    def unitLabels(self):
         # return ['Beam energy (GeV)','Beam current (mA)','Period (cm)','Number of periods','Kx','Ky','GAP (mm)','Min energy (eV)','Max energy (eV)','Number of energy steps','Distance (m)','X-pos. (mm)','Y-pos. (mm)','X slit [mm or mrad]','Y slit [mm or mrad]','Integration points X','Integration points Y']
         return ['Beam energy (GeV)','Beam current (mA)','Period (cm)','Number of periods','Ky','GAP (mm)','Min energy (eV)','Max energy (eV)','Number of energy steps','Distance (m)','X-pos. (mm)','Y-pos. (mm)','X slit [mm or mrad]','Y slit [mm or mrad]','Integration points X','Integration points Y']

    def unitFlags(self):
         return ['True','True','True','True','True','True','True','True','True','True','True','True','True','True','True','True']

    def get_help_name(self):
        return 'ID19ws'

    def check_fields(self):


        self.ENERGY = congruence.checkStrictlyPositiveNumber(self.ENERGY, "Beam Energy")
        self.CUR = congruence.checkStrictlyPositiveNumber(self.CUR, "Beam Current")
        self.PERIOD = congruence.checkStrictlyPositiveNumber(self.PERIOD, "Period")
        self.N = congruence.checkStrictlyPositiveNumber(self.N, "Number of Periods")
        self.KX = congruence.checkNumber(self.KX, "Kx")
        self.KY = congruence.checkPositiveNumber(self.KY, "Ky")
        self.GAP = congruence.checkPositiveNumber(self.GAP, "Gap")
        self.EMIN = congruence.checkPositiveNumber(self.EMIN, "Min Energy")
        self.EMAX = congruence.checkStrictlyPositiveNumber(self.EMAX, "Max Energy")
        congruence.checkLessThan(self.EMIN, self.EMAX, "Min Energy", "Max Energy")
        self.NEE = congruence.checkStrictlyPositiveNumber(self.NEE, "Number of energy steps")
        self.D = congruence.checkPositiveNumber(self.D, "Distance")
        self.XPC = congruence.checkNumber(self.XPC, "X-pos")
        self.YPC = congruence.checkNumber(self.YPC, "Y-pos")
        self.XPS = congruence.checkNumber(self.XPS, "X slit")
        self.YPS = congruence.checkNumber(self.YPS, "Y Slit")
        self.NXP = congruence.checkStrictlyPositiveNumber(self.NXP, "Integration points X")
        self.NYP = congruence.checkStrictlyPositiveNumber(self.NYP, "Integration points Y")


    def do_xoppy_calculation(self):
        return xoppy_calc_ws(ENERGY=self.ENERGY,CUR=self.CUR,PERIOD=self.PERIOD,N=self.N,KX=self.KX,KY=self.KY,EMIN=self.EMIN,EMAX=self.EMAX,NEE=self.NEE,D=self.D,XPC=self.XPC,YPC=self.YPC,XPS=self.XPS,YPS=self.YPS,NXP=self.NXP,NYP=self.NYP)

    def get_data_exchange_widget_name(self):
        return "WS"  

    def getTitles(self):
        return ['Flux','Spectral power','Cumulated power']

    def getXTitles(self):
        return ["Energy [eV]","Energy [eV]","Energy [eV]"]

    def getYTitles(self):
        return ["Flux [Photons/sec/0.1%bw]","Spectral Power [W/eV]","Cumulated Power [W]"]

    def getVariablesToPlot(self):
        return [(0, 1),(0, 2),(0, 3)]

    def getLogPlot(self):
        return [(True, True),(True, True),(False, False)]

    def receive_syned_data(self, data):

        if isinstance(data, synedb.Beamline):
            if not data._light_source is None and isinstance(data._light_source._magnetic_structure, synedid.InsertionDevice):
                light_source = data._light_source

                self.N = int(light_source._magnetic_structure._number_of_periods)
                self.ENERGY = light_source._electron_beam._energy_in_GeV
                self.CUR = 1e3*light_source._electron_beam._current
                self.PERIOD = 1e2*light_source._magnetic_structure._period_length
                self.KY = light_source._magnetic_structure._K_vertical
                self.GAP=(-1/0.02677)*numpy.log(self.KY/(93.4*2.89408*self.PERIOD*0.01))
                self.set_enabled(False)
            else:
                self.set_enabled(True)
        else:
            self.set_enabled(True)

    def set_enabled(self,value):
        if value == True:
                self.id_N.setEnabled(True)
                self.id_ENERGY.setEnabled(True)
                self.id_CUR.setEnabled(True)
                self.id_PERIOD.setEnabled(True)
                self.id_KY.setEnabled(True)
                self.id_GAP.setEnabled(True)
        else:
                self.id_N.setEnabled(False)
                self.id_ENERGY.setEnabled(False)
                self.id_CUR.setEnabled(False)
                self.id_PERIOD.setEnabled(False)
                self.id_KY.setEnabled(False)
                self.id_GAP.setEnabled(False)


# --------------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------------


def xoppy_calc_ws(ENERGY=6.0,CUR=200.0,PERIOD=15.0,N=28.0,KX=0.0,KY=18.34,\
                  EMIN=1000.0,EMAX=100000.0,NEE=2000,D=30.0,XPC=0.0,YPC=0.0,XPS=2.0,YPS=2.0,NXP=10,NYP=10):
    print("Inside xoppy_calc_ws. ")

    try:
        with open("ws.inp","wt") as f:
            f.write("inputs from xoppy \n")
            f.write("%f     %f\n"%(ENERGY,CUR))
            f.write("%f  %d  %f  %f\n"%(PERIOD,N,KX,KY))
            f.write("%f  %f   %d\n"%(EMIN,EMAX,NEE))
            f.write("%f  %f  %f  %f  %f  %d  %d\n"%(D,XPC,YPC,XPS,YPS,NXP,NYP))
            f.write("%d  \n"%(4))

        
        if platform.system() == "Windows":
            command = os.path.join(locations.home_bin(),'ws.exe')
        else:
            command = "'" + os.path.join(locations.home_bin(),'ws') + "'"
        print("Running command '%s' in directory: %s \n"%(command, locations.home_bin_run()))
        # TODO try to capture the text output of the external code
        os.system(command)

        # write spec file
        txt = open("ws.out").readlines()
        outFile = os.path.join(locations.home_bin_run(), "ws.spec")
        f = open(outFile,"w")

        f.write("#F ws.spec\n")
        f.write("\n")
        f.write("#S 1 ws results\n")
        f.write("#N 4\n")
        f.write("#L  Energy(eV)  Flux(photons/s/0.1%bw)  Spectral power(W/eV)  Cumulated power(W)\n")
        cum = 0.0
        estep = (EMAX-EMIN)/(NEE-1)
        for i in txt:
            tmp = i.strip(" ")
            if tmp[0].isdigit():
                tmp1 = numpy.fromstring(tmp, dtype=float, sep=' ')
                cum += tmp1[1]*codata.e*1e3
                f.write("%f  %g  %f  %f \n"%(tmp1[0],tmp1[1],tmp1[1]*codata.e*1e3,cum*estep))
            else:
               f.write("#UD "+tmp)
        f.close()
        print("File written to disk: ws.spec")

        # print output file
        # for line in txt:
        #     print(line, end="")
        print("Results written to file: %s"%os.path.join(locations.home_bin_run(),'ws.out'))

        return outFile
    except Exception as e:
        raise e



if __name__ == "__main__":
    import os
    os.environ['LD_LIBRARY_PATH'] = ''
    app = QApplication(sys.argv)
    w = OWID19ws()
    w.show()
    app.exec()
    w.saveSettings()



import numpy

from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QMessageBox, QApplication
from PyQt5.QtCore import QRect
from PyQt5.QtGui import QTextCursor

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui

from oasys.widgets.widget import AutomaticWidget

from silx.gui.plot import Plot2D
from orangecontrib.wofry.util.wofry_util import ImageViewWithFWHM

from orangecontrib.wofry.widgets.gui.python_script import PythonScript


class WofryWidget(AutomaticWidget):
    maintainer = "Luca Rebuffi"
    maintainer_email = "luca.rebuffi(@at@)anl.gov"

    IMAGE_WIDTH = 760
    IMAGE_HEIGHT = 545
    MAX_WIDTH = 1320
    MAX_HEIGHT = 705
    CONTROL_AREA_WIDTH = 410
    TABS_AREA_HEIGHT = 545

    want_main_area = 1

    view_type=Setting(1)

    def __init__(self, is_automatic=True, show_view_options=True, show_script_tab=True):
        super().__init__(is_automatic)

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width()*0.05),
                               round(geom.height()*0.05),
                               round(min(geom.width()*0.98, self.MAX_WIDTH)),
                               round(min(geom.height()*0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)

        self.main_tabs = oasysgui.tabWidget(self.mainArea)
        plot_tab = oasysgui.createTabPage(self.main_tabs, "Results")
        out_tab = oasysgui.createTabPage(self.main_tabs, "Output")
        #
        # add script tab to tabs panel
        #
        if show_script_tab:
            script_tab = oasysgui.createTabPage(self.main_tabs, "Script")
            self.wofry_script = PythonScript()
            self.wofry_script.code_area.setFixedHeight(400)
            script_box = gui.widgetBox(script_tab, "Python script", addSpace=True, orientation="horizontal")
            script_box.layout().addWidget(self.wofry_script)

        if show_view_options == True:
            view_box = oasysgui.widgetBox(plot_tab, "Results Options", addSpace=False, orientation="horizontal")
            view_box_1 = oasysgui.widgetBox(view_box, "", addSpace=False, orientation="vertical", width=350)

            self.view_type_combo = gui.comboBox(view_box_1, self, "view_type", label="View Results",
                                                labelWidth=220,
                                                items=["No", "Yes (image)","Yes (image + hist.)"],
                                                callback=self.set_ViewType, sendSelectedValue=False, orientation="horizontal")
        else:
            self.view_type = 1

        self.tab = []
        self.tabs = oasysgui.tabWidget(plot_tab)

        self.initializeTabs()

        self.set_ViewType()

        self.wofry_output = oasysgui.textArea(height=600, width=600)

        out_box = gui.widgetBox(out_tab, "System Output", addSpace=True, orientation="horizontal")
        out_box.layout().addWidget(self.wofry_output)

        gui.rubber(self.mainArea)


    def initializeTabs(self):
        raise NotImplementedError()

    def set_ViewType(self):
        self.progressBarInit()

        try:
            self.initializeTabs()
            self.plot_results()
        except Exception as exception:
            QtWidgets.QMessageBox.critical(self, "Error",
                                       str(exception),
                QtWidgets.QMessageBox.Ok)

            if self.IS_DEVELOP: raise exception

        self.progressBarFinished()

    def plot_results(self, progressBarValue=80):
        if not self.view_type == 0:
            self.do_plot_results(progressBarValue)


    def do_plot_results(self, progressBarValue):
        raise NotImplementedError()

    def plot_data1D(self, x, y, progressBarValue, tabs_canvas_index, plot_canvas_index, title="", xtitle="", ytitle="",
                    log_x=False, log_y=False, color='blue', replace=True, control=False, calculate_fwhm=True,
                    xrange=None, yrange=None, symbol=''):

        if tabs_canvas_index is None: tabs_canvas_index = 0 #back compatibility?


        self.tab[tabs_canvas_index].layout().removeItem(self.tab[tabs_canvas_index].layout().itemAt(0))

        self.plot_canvas[plot_canvas_index] = oasysgui.plotWindow(parent=None,
                                                                  backend=None,
                                                                  resetzoom=True,
                                                                  autoScale=False,
                                                                  logScale=True,
                                                                  grid=True,
                                                                  curveStyle=True,
                                                                  colormap=False,
                                                                  aspectRatio=False,
                                                                  yInverted=False,
                                                                  copy=True,
                                                                  save=True,
                                                                  print_=True,
                                                                  control=control,
                                                                  position=True,
                                                                  roi=False,
                                                                  mask=False,
                                                                  fit=False)


        self.plot_canvas[plot_canvas_index].setDefaultPlotLines(True)
        self.plot_canvas[plot_canvas_index].setActiveCurveColor(color='blue')
        self.plot_canvas[plot_canvas_index].setGraphXLabel(xtitle)
        self.plot_canvas[plot_canvas_index].setGraphYLabel(ytitle)

        # ALLOW FIT BUTTON HERE
        self.plot_canvas[plot_canvas_index].fitAction.setVisible(True)

        # overwrite FWHM and peak values
        if calculate_fwhm:
            try:
                t = numpy.where(y>=max(y)*0.5)
                x_left,x_right =  x[t[0][0]], x[t[0][-1]]

                self.plot_canvas[plot_canvas_index].addMarker(x_left, 0.5*y.max(), legend="G1",
                                                              text="FWHM=%5.2f"%(numpy.abs(x_right-x_left)),
                                                              color="pink",selectable=False, draggable=False,
                                                              symbol="+", constraint=None)
                self.plot_canvas[plot_canvas_index].addMarker(x_right, 0.5*y.max(), legend="G2", text=None, color="pink",
                                                              selectable=False, draggable=False, symbol="+", constraint=None)
            except:
                pass

        self.tab[tabs_canvas_index].layout().addWidget(self.plot_canvas[plot_canvas_index])

        WofryWidget.plot_histo(self.plot_canvas[plot_canvas_index], x, y, title, xtitle, ytitle, color, replace, symbol=symbol)

        self.plot_canvas[plot_canvas_index].setXAxisLogarithmic(log_x)
        self.plot_canvas[plot_canvas_index].setYAxisLogarithmic(log_y)


        if xrange is not None:
            self.plot_canvas[plot_canvas_index].setGraphXLimits(xrange[0],xrange[1])
        if yrange is not None:
            self.plot_canvas[plot_canvas_index].setGraphYLimits(yrange[0],yrange[1])

        if min(y) < 0:
            if log_y:
                self.plot_canvas[plot_canvas_index].setGraphYLimits(min(y)*1.2, max(y)*1.2)
            else:
                self.plot_canvas[plot_canvas_index].setGraphYLimits(min(y)*1.01, max(y)*1.01)
        else:
            if log_y:
                self.plot_canvas[plot_canvas_index].setGraphYLimits(min(y), max(y)*1.2)
            else:
                self.plot_canvas[plot_canvas_index].setGraphYLimits(min(y)*0.99, max(y)*1.01)

        self.progressBarSet(progressBarValue)

    def plot_data2D(self, data2D, dataX, dataY, progressBarValue, tabs_canvas_index, plot_canvas_index,
                    title="",xtitle="", ytitle=""):

        if self.view_type == 0:
            pass
        elif self.view_type == 1:
            self.plot_data2D_only_image(data2D, dataX, dataY, progressBarValue, tabs_canvas_index,plot_canvas_index,
                         title=title, xtitle=xtitle, ytitle=ytitle)
        elif self.view_type == 2:
            self.plot_data2D_with_histograms(data2D, dataX, dataY, progressBarValue, tabs_canvas_index,plot_canvas_index,
                         title=title, xtitle=xtitle, ytitle=ytitle)

    def plot_data2D_only_image(self, data2D, dataX, dataY, progressBarValue, tabs_canvas_index, plot_canvas_index,
                    title="", xtitle="", ytitle=""):

        self.tab[tabs_canvas_index].layout().removeItem(self.tab[tabs_canvas_index].layout().itemAt(0))

        xmin = dataX[0] # numpy.min(dataX)
        xmax = dataX[-1] # numpy.max(dataX)
        ymin = dataY[0] # numpy.min(dataY)
        ymax = dataY[-1] # numpy.max(dataY)

        origin = (xmin, ymin)
        scale = (abs((xmax-xmin)/len(dataX)), abs((ymax-ymin)/len(dataY)))

        data_to_plot = data2D.T

        colormap = {"name":"temperature", "normalization":"linear", "autoscale":True, "vmin":0, "vmax":0, "colors":256}

        self.plot_canvas[plot_canvas_index] = Plot2D()

        self.plot_canvas[plot_canvas_index].resetZoom()
        self.plot_canvas[plot_canvas_index].setXAxisAutoScale(True)
        self.plot_canvas[plot_canvas_index].setYAxisAutoScale(True)
        self.plot_canvas[plot_canvas_index].setGraphGrid(False)
        self.plot_canvas[plot_canvas_index].setKeepDataAspectRatio(True)
        self.plot_canvas[plot_canvas_index].yAxisInvertedAction.setVisible(False)

        self.plot_canvas[plot_canvas_index].setXAxisLogarithmic(False)
        self.plot_canvas[plot_canvas_index].setYAxisLogarithmic(False)
        #silx 0.4.0
        self.plot_canvas[plot_canvas_index].getMaskAction().setVisible(False)
        self.plot_canvas[plot_canvas_index].getRoiAction().setVisible(False)
        self.plot_canvas[plot_canvas_index].getColormapAction().setVisible(True)
        self.plot_canvas[plot_canvas_index].setKeepDataAspectRatio(False)

        self.plot_canvas[plot_canvas_index].addImage(numpy.array(data_to_plot),
                                                     legend="None",
                                                     scale=scale,
                                                     origin=origin,
                                                     colormap=colormap,
                                                     replace=True)


        self.plot_canvas[plot_canvas_index].setGraphXLabel(xtitle)
        self.plot_canvas[plot_canvas_index].setGraphYLabel(ytitle)
        self.plot_canvas[plot_canvas_index].setGraphTitle(title)

        self.tab[tabs_canvas_index].layout().addWidget(self.plot_canvas[plot_canvas_index])

        self.progressBarSet(progressBarValue)

    def plot_data2D_with_histograms(self, data2D, dataX, dataY, progressBarValue, tabs_canvas_index, plot_canvas_index,
                                    title="", xtitle="", ytitle=""):

        xum = "H [\u03BCm]"
        yum = "V [\u03BCm]"

        self.tab[tabs_canvas_index].layout().removeItem(self.tab[tabs_canvas_index].layout().itemAt(0))

        data_to_plot = data2D

        self.plot_canvas[plot_canvas_index] = ImageViewWithFWHM() #Plot2D()

        colormap = {"name":"temperature", "normalization":"linear", "autoscale":True, "vmin":0, "vmax":0, "colors":256}

        self.plot_canvas[plot_canvas_index].plot_2D(numpy.array(data_to_plot),dataX,dataY,factor1=1e0,factor2=1e0,
               title=title,xtitle=xtitle, ytitle=ytitle,xum=xum,yum=yum,colormap=colormap)


        self.tab[tabs_canvas_index].layout().addWidget(self.plot_canvas[plot_canvas_index])

        self.progressBarSet(progressBarValue)

    @classmethod
    def plot_histo(cls, plot_window, x, y, title, xtitle, ytitle, color='blue', replace=True, symbol=''):
        import matplotlib
        matplotlib.rcParams['axes.formatter.useoffset']='False'

        plot_window.addCurve(x, y, title, symbol=symbol, color=color, xlabel=xtitle, ylabel=ytitle, replace=replace) #'+', '^', ','

        if not xtitle is None: plot_window.setGraphXLabel(xtitle)
        if not ytitle is None: plot_window.setGraphYLabel(ytitle)
        if not title is None: plot_window.setGraphTitle(title)

        plot_window.resetZoom()
        plot_window.replot()

        plot_window.setActiveCurve(title)

    def writeStdOut(self, text):
        cursor = self.wofry_output.textCursor()
        cursor.movePosition(QTextCursor.End)
        cursor.insertText(text)
        self.wofry_output.setTextCursor(cursor)
        self.wofry_output.ensureCursorVisible()



__author__ = 'labx'

import numpy


from orangecontrib.wofry.util.wofry_objects import WofryData
from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget


import sys

from orangewidget.widget import OWAction
from oasys.widgets import gui as oasysgui
from oasys.widgets.gui import ConfirmDialog

from orangewidget import gui
from PyQt5.QtGui import QFont, QPalette, QColor
from PyQt5.QtWidgets import QMessageBox
from orangewidget.settings import Setting

from oasys.util.oasys_util import TriggerIn, TriggerOut, EmittingStream

from oasys.util.oasys_util import get_fwhm

class OWstepper1D(WofryWidget):

    name = "Loop stepper 1D"
    id = "LoopStepper1D"
    description = "Loop stepper 1D"
    icon = "icons/stepper.png"
    priority = 160

    category = "Wofry Tools"
    keywords = ["data", "file", "load", "read"]

    inputs = [("WofryData", WofryData, "set_input"),
              ("Trigger", TriggerIn, "passTrigger")]

    outputs = [{"name":"Trigger",
                "type":TriggerOut,
                "doc":"Trigger",
                "id":"Trigger"}]

    #
    # trigger
    #
    current_new_object = 0
    run_loop = True
    suspend_loop = False

    variable_name = Setting("radius")
    variable_display_name = Setting("<variable display name>")
    variable_um = Setting("<u.m.>")

    variable_value_from = Setting(1000.0)
    variable_value_to = Setting(1100.0)
    number_of_new_objects = Setting(100)
    variable_value_step = Setting(100.0)
    accumulate_flag = Setting(0)

    current_variable_value = None

    #################################
    process_last = True
    #################################

    #
    # viewer
    #
    wofry_data = None
    accumulated_data = None

    titles = ["Intensity (current/last run)","Intensity (accumulated)",
              "Scanned peak", "Scanned FWHM", "Scanned Integral", "Scans"]

    def __init__(self):
        super().__init__(is_automatic=False, show_view_options=True)  #<<<<<<<<<<<<<<<<<<<

        #
        #
        #
        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)
        tabs_setting = oasysgui.tabWidget(self.controlArea)
        tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT + 50)
        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)

        #=====================================================================================
        # trigger
        #=====================================================================================

        #
        # trigger
        #
        self.runaction = OWAction("Start", self)
        self.runaction.triggered.connect(self.startLoop)
        self.addAction(self.runaction)

        self.runaction = OWAction("Stop", self)
        self.runaction.triggered.connect(self.stopLoop)
        self.addAction(self.runaction)

        self.runaction = OWAction("Suspend", self)
        self.runaction.triggered.connect(self.suspendLoop)
        self.addAction(self.runaction)

        self.runaction = OWAction("Restart", self)
        self.runaction.triggered.connect(self.restartLoop)
        self.addAction(self.runaction)



        self.tab_trig = oasysgui.createTabPage(tabs_setting, "Stepper/Trigger")

        #
        #

        button_box = oasysgui.widgetBox(self.tab_trig, "", addSpace=True, orientation="horizontal")

        self.start_button = gui.button(button_box, self, "Start", callback=self.startLoop)
        self.start_button.setFixedHeight(35)

        stop_button = gui.button(button_box, self, "Stop", callback=self.stopLoop)
        stop_button.setFixedHeight(35)
        font = QFont(stop_button.font())
        font.setBold(True)
        stop_button.setFont(font)
        palette = QPalette(stop_button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('red'))
        stop_button.setPalette(palette) # assign new palette

        self.stop_button = stop_button

        button_box = oasysgui.widgetBox(self.tab_trig, "", addSpace=True, orientation="horizontal")

        suspend_button = gui.button(button_box, self, "Suspend", callback=self.suspendLoop)
        suspend_button.setFixedHeight(35)
        font = QFont(suspend_button.font())
        font.setBold(True)
        suspend_button.setFont(font)
        palette = QPalette(suspend_button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('orange'))
        suspend_button.setPalette(palette) # assign new palette

        self.re_start_button = gui.button(button_box, self, "Restart", callback=self.restartLoop)
        self.re_start_button.setFixedHeight(35)
        self.re_start_button.setEnabled(False)

        left_box_1 = oasysgui.widgetBox(self.tab_trig, "Loop Management", addSpace=True, orientation="vertical", width=385, height=220) # 320

        oasysgui.lineEdit(left_box_1, self, "variable_name", "Variable Name", labelWidth=100, valueType=str, orientation="horizontal")

        gui.separator(left_box_1)

        oasysgui.lineEdit(left_box_1, self, "variable_value_from", "Value From", labelWidth=250, valueType=float, orientation="horizontal", callback=self.calculate_step)
        oasysgui.lineEdit(left_box_1, self, "variable_value_to", "Value to", labelWidth=250, valueType=float, orientation="horizontal", callback=self.calculate_step)
        oasysgui.lineEdit(left_box_1, self, "number_of_new_objects", "Number of Steps", labelWidth=250, valueType=int, orientation="horizontal", callback=self.calculate_step)
        oasysgui.lineEdit(left_box_1, self, "variable_value_step", "Step Value", labelWidth=250, valueType=float, orientation="horizontal", callback=self.calculate_step2)




        left_box_2 = oasysgui.widgetBox(self.tab_trig, "Advancement", addSpace=True, orientation="vertical", width=385, height=90) # 320

        self.le_current_new_object = oasysgui.lineEdit(left_box_2, self, "current_new_object", "Current New " + self.get_object_name(), labelWidth=250, valueType=int, orientation="horizontal")
        self.le_current_new_object.setReadOnly(True)
        font = QFont(self.le_current_new_object.font())
        font.setBold(True)
        self.le_current_new_object.setFont(font)
        palette = QPalette(self.le_current_new_object.palette()) # make a copy of the palette
        palette.setColor(QPalette.Text, QColor('dark blue'))
        palette.setColor(QPalette.Base, QColor(243, 240, 160))
        self.le_current_new_object.setPalette(palette)

        self.le_current_new_object = oasysgui.lineEdit(left_box_2, self, "current_variable_value", "Current Variable Value", labelWidth=250, valueType=float, orientation="horizontal")
        self.le_current_new_object.setReadOnly(True)
        font = QFont(self.le_current_new_object.font())
        font.setBold(True)
        self.le_current_new_object.setFont(font)
        palette = QPalette(self.le_current_new_object.palette()) # make a copy of the palette
        palette.setColor(QPalette.Text, QColor('dark blue'))
        palette.setColor(QPalette.Base, QColor(243, 240, 160))
        self.le_current_new_object.setPalette(palette)


        #=====================================================================================
        # Wavefront
        #=====================================================================================
        display_control_box = oasysgui.widgetBox(self.tab_trig, "Plots", addSpace=True, orientation="vertical") #, width=385, height=120)

        button_box = oasysgui.widgetBox(display_control_box, "", addSpace=False, orientation="horizontal") #, width=385, height=120)
        gui.separator(button_box)
        button = gui.button(button_box, self, "Refresh", callback=self.refresh)
        font = QFont(button.font())
        font.setBold(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Blue'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)

        gui.separator(button_box)

        button = gui.button(button_box, self, "Clear", callback=self.reset_accumumation)
        font = QFont(button.font())
        font.setBold(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Blue'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)

        left_box_3 = oasysgui.widgetBox(display_control_box, "", addSpace=True, orientation="vertical") #, width=385, height=120)

        oasysgui.lineEdit(left_box_3, self, "variable_display_name", "Variable Display Name", labelWidth=175, valueType=str, orientation="horizontal")
        oasysgui.lineEdit(left_box_3, self, "variable_um", "Variable Units", labelWidth=250, valueType=str, orientation="horizontal")

        gui.comboBox(left_box_3, self, "accumulate_flag", label="Display scans:", labelWidth=350,
                     items=["current", "accumulated"],
                     sendSelectedValue=False, orientation="horizontal",
                     callback=self.replot_after_accumulate_change)

        self.calculate_step()

    def replot_after_accumulate_change(self):
        if self.view_type != 0:
            self.progressBarInit()
            self.plot_results()
            self.progressBarFinished

    def initializeTabs(self):
        size = len(self.titles) # len(self.tab)
        indexes = range(0, size)

        for index in indexes:
            self.tabs.removeTab(size-1-index)

        self.tab = []
        self.plot_canvas = []


        for i in range(size):
            self.tab.append(gui.createTabPage(self.tabs, self.titles[i]))
            self.plot_canvas.append(None)

        for tab in self.tab:
            tab.setFixedHeight(self.IMAGE_HEIGHT)
            tab.setFixedWidth(self.IMAGE_WIDTH)


    def set_input(self, wofry_data):
        if not wofry_data is None:
            self.wofry_data = wofry_data

            if self.accumulated_data is None:
                self.accumulated_data = {}
                self.accumulated_data["counter"] = 1
                self.accumulated_data["intensity"] = self.wofry_data.get_wavefront().get_intensity()

                intensities = []
                intensities.append( self.wofry_data.get_wavefront().get_intensity() )
                self.accumulated_data["intensities"] = intensities

                current_variable_values = []
                current_variable_values.append(self.current_variable_value)
                self.accumulated_data["current_variable_values"] = current_variable_values

                self.accumulated_data["complex_amplitude"] = self.wofry_data.get_wavefront().get_complex_amplitude()

                self.accumulated_data["x"] = self.wofry_data.get_wavefront().get_abscissas()

            else:
                self.accumulated_data["counter"] += 1
                self.accumulated_data["intensity"] = self.wofry_data.get_wavefront().get_intensity()

                intensities = self.accumulated_data["intensities"]
                intensities.append( self.wofry_data.get_wavefront().get_intensity() )
                self.accumulated_data["intensities"] = intensities

                current_variable_values = self.accumulated_data["current_variable_values"]
                current_variable_values.append(self.current_variable_value)
                self.accumulated_data["current_variable_values"] = current_variable_values

                self.accumulated_data["complex_amplitude"] += self.wofry_data.get_wavefront().get_complex_amplitude()


            self.refresh()
            #NEWNEWNEW
            self.passTrigger(TriggerIn(new_object=True))

    def refresh(self):

        self.wofry_output.setText("")
        sys.stdout = EmittingStream(textWritten=self.writeStdOut)
        try:
            if self.wofry_data is not None:
                if self.current_variable_value is not None:
                    print("> Running %s (%s) = " % (self.variable_name, self.variable_display_name), self.current_variable_value)

                if self.view_type != 0:
                    self.progressBarInit()
                    current_index = self.tabs.currentIndex()
                    self.initializeTabs()
                    self.plot_results()
                    self.tabs.setCurrentIndex(current_index)
                    self.progressBarFinished()
        except Exception as exception:
            QMessageBox.critical(self, "Error", str(exception), QMessageBox.Ok)


    def do_plot_results(self, progressBarValue=50):
        if self.accumulated_data is None:
            return
        else:

            self.progressBarSet(progressBarValue)

            self.plot_data1D(x=1e6*self.accumulated_data["x"],
                             y=self.accumulated_data["intensity"],
                             progressBarValue=progressBarValue + 5,
                             tabs_canvas_index=0,
                             plot_canvas_index=0,
                             title=self.titles[0],
                             calculate_fwhm=True,
                             xtitle="Spatial Coordinate [$\mu$m]",
                             ytitle="Intensity")

            try:
                nruns = len(self.accumulated_data['intensities'])

                if nruns == 1:
                    x = numpy.arange(nruns)
                else:
                    x = numpy.array(self.accumulated_data["current_variable_values"])

                peak = numpy.zeros(nruns)
                fwhm = numpy.zeros(nruns)
                integral = numpy.zeros(nruns)
                all_scans = numpy.zeros((nruns, len(self.accumulated_data['intensities'][0])))
                for i in range(nruns):
                    current_profile = self.accumulated_data['intensities'][i]
                    if i == 0:
                        accumulated_profile = current_profile.copy()
                    else:
                        accumulated_profile += current_profile


                    if self.accumulate_flag == 0:
                        active_profile = current_profile
                    elif self.accumulate_flag == 1:
                        active_profile = accumulated_profile

                    peak[i] = numpy.max(active_profile)
                    try:
                        fwhm[i], _, _ = get_fwhm(active_profile, self.accumulated_data['x'] )
                        print("fwhm: ", i, fwhm[i])
                    except:
                        fwhm[i] = 0.0
                        print("bad fwhm: ",i,fwhm[i])
                    integral[i] = (active_profile).sum()
                    all_scans[i, :] = active_profile
            except:
                x = numpy.array([-2,-1])
                peak = numpy.array([-1, -1])
                fwhm = numpy.array([-1, -1])

            self.plot_data1D(x=1e6*self.accumulated_data["x"],
                             y=accumulated_profile,
                             progressBarValue=progressBarValue + 5,
                             tabs_canvas_index=1,
                             plot_canvas_index=1,
                             title=self.titles[1],
                             calculate_fwhm=True,
                             xtitle="Spatial Coordinate [$\mu$m]",
                             ytitle="Intensity")

            self.plot_data1D(x=x,
                             y=peak,
                             progressBarValue=progressBarValue + 5,
                             tabs_canvas_index=2,
                             plot_canvas_index=2,
                             calculate_fwhm=False,
                             title="%s nruns: %s" % (self.titles[2] , self.accumulated_data["counter"]),
                             xtitle="%s [%s]" % (self.variable_display_name, self.variable_um),
                             ytitle="Peak Intensity")

            self.plot_data1D(x=x,
                             y=1e6*fwhm,
                             progressBarValue=progressBarValue + 5,
                             tabs_canvas_index=3,
                             plot_canvas_index=3,
                             calculate_fwhm=False,
                             title="%s nruns: %s" % (self.titles[3] , len(self.accumulated_data["intensities"])),
                             xtitle="%s [%s]" % (self.variable_display_name, self.variable_um),
                             ytitle="FWHM [um]")

            self.plot_data1D(x=x,
                             y=integral,
                             progressBarValue=progressBarValue + 5,
                             tabs_canvas_index=4,
                             plot_canvas_index=4,
                             calculate_fwhm=False,
                             title="%s nruns: %s" % (self.titles[4] , len(self.accumulated_data["intensities"])),
                             xtitle="%s [%s]" % (self.variable_display_name, self.variable_um),
                             ytitle="Integral [a.u.]")

            if nruns > 1:
                self.plot_data2D(all_scans, x, 1e6*self.accumulated_data["x"],
                                 progressBarValue=progressBarValue + 5,
                                 tabs_canvas_index=5,
                                 plot_canvas_index=5,
                                 title="%s nruns: %s" % (self.titles[5] , len(self.accumulated_data["intensities"])),
                                 xtitle="%s [%s]" % (self.variable_display_name, self.variable_um),
                                 ytitle="Spatial Coordinate [$\mu$m]",)

            print("> current value: ",self.current_variable_value)
            print("> current valueSS: ", self.accumulated_data["current_variable_values"])
            print("> current new object: ",self.current_new_object)
            print("> self.variable_name = ", self.variable_name)
            print("> self.variable_display_name = ",self.variable_display_name)
            print("> self.current_variable_value = ",self.current_variable_value)
            print("> self.variable_um = ",self.variable_um)




    def reset_accumumation(self):
        self.initializeTabs()
        self.accumulated_data = None
        self.wofry_data = None

    #
    # trigger methods
    #

    def calculate_step(self):
        if self.number_of_new_objects != 0:
            self.variable_value_step = round(
                (self.variable_value_to - self.variable_value_from) / self.number_of_new_objects, 8)

    def calculate_step2(self):
        self.number_of_new_objects = int((self.variable_value_to - self.variable_value_from) / \
                                self.variable_value_step)
        self.calculate_step()

    def startLoop(self):

        self.reset_accumumation()  #NEWNEW

        self.current_new_object = 1

        do_loop = True

        self.current_variable_value = round(self.variable_value_from, 8)
        self.calculate_step()

        if do_loop:
            self.start_button.setEnabled(False)

            self.setStatusMessage(
                "Running " + self.get_object_name() + " " + str(self.current_new_object) + " of " + str(
                    self.number_of_new_objects))
            self.send("Trigger", TriggerOut(new_object=True, additional_parameters={"variable_name": self.variable_name,
                                                                                    "variable_display_name": self.variable_display_name,
                                                                                    "variable_value": self.current_variable_value,
                                                                                    "variable_um": self.variable_um}))

    def stopLoop(self):
        if ConfirmDialog.confirmed(parent=self, message="Confirm Interruption of the Loop?"):
            self.run_loop = False
            self.current_variable_value = None
            self.setStatusMessage("Interrupted by user")

    def suspendLoop(self):
        try:
            if ConfirmDialog.confirmed(parent=self, message="Confirm Suspension of the Loop?"):
                self.run_loop = False
                self.suspend_loop = True
                self.stop_button.setEnabled(False)
                self.re_start_button.setEnabled(True)
                self.setStatusMessage("Suspended by user")
        except:
            pass

    def restartLoop(self):
        try:
            self.run_loop = True
            self.suspend_loop = False
            self.stop_button.setEnabled(True)
            self.re_start_button.setEnabled(False)
            self.passTrigger(TriggerIn(new_object=True))
        except:
            pass

    def passTrigger(self, trigger):

        if self.run_loop:
            if trigger:
                if trigger.interrupt:
                    self.current_new_object = 0
                    self.current_variable_value = None
                    self.start_button.setEnabled(True)
                    self.setStatusMessage("")
                    self.send("Trigger", TriggerOut(new_object=False))
                elif trigger.new_object:
                    if self.current_new_object == 0:
                        QMessageBox.critical(self, "Error", "Loop has to be started properly: press the button Start",
                                             QMessageBox.Ok)
                        return

                    if (self.current_new_object < self.number_of_new_objects) or (
                            self.current_new_object == self.number_of_new_objects):
                        if self.current_variable_value is None:
                            self.current_new_object = 1

                            self.current_variable_value = round(self.variable_value_from, 8)
                            self.calculate_step()
                        else:
                            self.current_new_object += 1
                            self.current_variable_value = round(
                                self.current_variable_value + self.variable_value_step, 8)

                        self.setStatusMessage(
                            "Running " + self.get_object_name() + " " + str(self.current_new_object) + " of " + str(
                                self.number_of_new_objects))
                        self.start_button.setEnabled(False)
                        self.send("Trigger", TriggerOut(new_object=True,
                                                        additional_parameters={"variable_name": self.variable_name,
                                                                               "variable_display_name": self.variable_display_name,
                                                                               "variable_value": self.current_variable_value,
                                                                               "variable_um": self.variable_um}))
                    else:
                        self.current_new_object = 0
                        self.current_variable_value = None
                        self.start_button.setEnabled(True)
                        self.setStatusMessage("")
                        self.send("Trigger", TriggerOut(new_object=False))
        else:
            if not self.suspend_loop:
                self.current_new_object = 0
                self.current_variable_value = None
                self.start_button.setEnabled(True)

            self.send("Trigger", TriggerOut(new_object=False))
            self.setStatusMessage("")
            self.run_loop = True
            self.suspend_loop = False

    def get_object_name(self):
        return "Object"


if __name__ == '__main__':

    from wofry.propagator.wavefront1D.generic_wavefront import GenericWavefront1D
    from PyQt5.QtWidgets import QApplication

    app = QApplication([])
    ow =OWstepper1D()

    wf = GenericWavefront1D.initialize_wavefront_from_arrays(numpy.linspace(-1e-3,1e-3,300),
                                                             numpy.linspace(-1e-3,1e-3,300)**2 )

    ow.set_input(WofryData(wavefront=wf))
    ow.show()
    app.exec_()
    ow.saveSettings()



import sys, numpy

from PyQt5.QtGui import QPalette, QColor, QFont
from PyQt5.QtWidgets import QMessageBox

from orangewidget import gui
from orangewidget import widget
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence
from oasys.widgets.gui import ConfirmDialog
from oasys.util.oasys_util import TriggerIn, EmittingStream

from syned.widget.widget_decorator import WidgetDecorator
from syned.beamline.element_coordinates import ElementCoordinates
from syned.beamline.beamline_element import BeamlineElement
from syned.beamline.shape import *

from wofry.propagator.propagator import PropagationManager, PropagationElements, PropagationParameters
from wofryimpl.propagator.propagators2D.fresnel import Fresnel2D
from wofryimpl.propagator.propagators2D.fresnel_convolution import FresnelConvolution2D
from wofryimpl.propagator.propagators2D.fraunhofer import Fraunhofer2D
from wofryimpl.propagator.propagators2D.integral import Integral2D
from wofryimpl.propagator.propagators2D.fresnel_zoom_xy import FresnelZoomXY2D

from wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D
from orangecontrib.wofry.util.wofry_objects import WofryData
from orangecontrib.esrf.wofry.widgets.gui.ow_wofry_widget import WofryWidget # TODO: from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget

from wofryimpl.beamline.beamline import WOBeamline

def initialize_default_propagator_2D():
    propagator = PropagationManager.Instance()

    propagator.add_propagator(Fraunhofer2D())
    propagator.add_propagator(Fresnel2D())
    propagator.add_propagator(FresnelConvolution2D())
    propagator.add_propagator(Integral2D())
    propagator.add_propagator(FresnelZoomXY2D())

try:
    initialize_default_propagator_2D()
except:
    pass

class OWWOOpticalElement(WofryWidget, WidgetDecorator):

    maintainer = "Luca Rebuffi"
    maintainer_email = "luca.rebuffi(@at@)elettra.eu"
    keywords = ["data", "file", "load", "read"]
    category = "Wofry Optical Elements"

    outputs = [
                # {"name":"GenericWavefront2D",
                # "type":GenericWavefront2D,
                # "doc":"GenericWavefront2D",
                # "id":"GenericWavefront2D"},
               {"name":"WofryData",
                "type":WofryData,
                "doc":"WofryData",
                "id":"WofryData"},
               {"name":"Trigger",
                "type": TriggerIn,
                "doc":"Feedback signal to start a new beam simulation",
                "id":"Trigger"}]

    inputs = [("WofryData", WofryData, "set_input"),
              # ("GenericWavefront2D", GenericWavefront2D, "set_input"),
              WidgetDecorator.syned_input_data()[0]]

    oe_name         = Setting("")
    p               = Setting(0.0)
    q               = Setting(0.0)
    angle_radial    = Setting(0.0)
    angle_azimuthal = Setting(0.0)

    shape = Setting(0)
    surface_shape = Setting(0)

    input_data = None
    wavefront_to_plot = None

    propagators_list = ["Fresnel", "Fresnel (Convolution)", "Fraunhofer", "Integral", "Fresnel Zoom XY"]

    propagator = Setting(4)
    shift_half_pixel = Setting(1)

    shuffle_interval = Setting(0)
    calculate_grid_only = Setting(1)
    magnification_x = Setting(1.0)
    magnification_y = Setting(1.0)

    def __init__(self, is_automatic=True, show_view_options=True, show_script_tab=False):
        super().__init__(is_automatic=is_automatic, show_view_options=show_view_options, show_script_tab=show_script_tab)

        self.runaction = widget.OWAction("Propagate Wavefront", self)
        self.runaction.triggered.connect(self.propagate_wavefront)
        self.addAction(self.runaction)

        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Propagate Wavefront", callback=self.propagate_wavefront)
        font = QFont(button.font())
        font.setBold(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Blue'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)

        button = gui.button(button_box, self, "Reset Fields", callback=self.callResetSettings)
        font = QFont(button.font())
        font.setItalic(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Red'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)
        button.setFixedWidth(150)

        gui.separator(self.controlArea)

        self.tabs_setting = oasysgui.tabWidget(self.controlArea)
        self.tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT)
        self.tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)

        self.tab_bas = oasysgui.createTabPage(self.tabs_setting, "O.E. Setting")
        self.tab_pro = oasysgui.createTabPage(self.tabs_setting, "Propagation Setting")

        oasysgui.lineEdit(self.tab_bas, self, "oe_name", "O.E. Name", labelWidth=260, valueType=str, orientation="horizontal")

        self.coordinates_box = oasysgui.widgetBox(self.tab_bas, "Coordinates", addSpace=True, orientation="vertical")

        oasysgui.lineEdit(self.coordinates_box, self, "p", "Distance from previous Continuation Plane [m]", labelWidth=280, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.coordinates_box, self, "q", "Distance to next Continuation Plane [m]", labelWidth=280, valueType=float, orientation="horizontal")
        # srio commented; TODO: implement it correctly
        # oasysgui.lineEdit(self.coordinates_box, self, "angle_radial", "Incident Angle (to normal) [deg]", labelWidth=280, valueType=float, orientation="horizontal")
        # oasysgui.lineEdit(self.coordinates_box, self, "angle_azimuthal", "Rotation along Beam Axis [deg]", labelWidth=280, valueType=float, orientation="horizontal")

        self.draw_specific_box()

        gui.comboBox(self.tab_pro, self, "propagator", label="Propagator", labelWidth=260,
                     items=self.propagators_list,
                     callback=self.set_Propagator,
                     sendSelectedValue=False, orientation="horizontal")

        self.fresnel_box = oasysgui.widgetBox(self.tab_pro, "", addSpace=False, orientation="vertical", height=90)

        gui.comboBox(self.fresnel_box, self, "shift_half_pixel", label="Shift Half Pixel", labelWidth=260,
                     items=["No", "Yes"],
                     sendSelectedValue=False, orientation="horizontal")

        self.fraunhofer_box = oasysgui.widgetBox(self.tab_pro, "", addSpace=False, orientation="vertical", height=90)

        gui.comboBox(self.fraunhofer_box, self, "shift_half_pixel", label="Shift Half Pixel", labelWidth=260,
                     items=["No", "Yes"],
                     sendSelectedValue=False, orientation="horizontal")

        self.integral_box = oasysgui.widgetBox(self.tab_pro, "", addSpace=False, orientation="vertical", height=90)


        oasysgui.lineEdit(self.integral_box, self, "shuffle_interval", "Shuffle Interval (0=no shift) [m]", labelWidth=260, valueType=float, orientation="horizontal")

        gui.comboBox(self.integral_box, self, "calculate_grid_only", label="Calculate Grid Only", labelWidth=260,
                     items=["No", "Yes"],
                     sendSelectedValue=False, orientation="horizontal")


        #new zoom
        self.zoom_box = oasysgui.widgetBox(self.tab_pro, "", addSpace=False, orientation="vertical", height=90)

        gui.comboBox(self.zoom_box, self, "shift_half_pixel", label="Shift Half Pixel", labelWidth=260,
                     items=["No", "Yes"],
                     sendSelectedValue=False, orientation="horizontal")

        oasysgui.lineEdit(self.zoom_box, self, "magnification_x", "Magnification X",
                          labelWidth=260, valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.zoom_box, self, "magnification_y", "Magnification Y",
                          labelWidth=260, valueType=float, orientation="horizontal")

        self.set_Propagator()

    def set_Propagator(self):
        self.fresnel_box.setVisible(self.propagator <= 1)
        self.fraunhofer_box.setVisible(self.propagator == 2)
        self.integral_box.setVisible(self.propagator == 3)
        self.zoom_box.setVisible(self.propagator == 4)

    def draw_specific_box(self):
        # raise NotImplementedError()
        pass

    def check_data(self):
        congruence.checkNumber(self.p, "Distance from previous Continuation Plane")
        congruence.checkNumber(self.q, "Distance to next Continuation Plane")
        congruence.checkAngle(self.angle_radial, "Incident Angle (to normal)")
        congruence.checkAngle(self.angle_azimuthal, "Rotation along Beam Axis")

    def propagate_wavefront(self):
        if True: #try:
            self.wofry_output.setText("")
            self.progressBarInit()

            sys.stdout = EmittingStream(textWritten=self.writeStdOut)

            if self.input_data is None: raise Exception("No Input Data")

            self.check_data()

            # propagation to o.e.

            input_wavefront  = self.input_data.get_wavefront()
            beamline         = self.input_data.get_beamline().duplicate()

            optical_element = self.get_optical_element()
            optical_element.name = self.oe_name if not self.oe_name is None else self.windowTitle()

            beamline_element = BeamlineElement(optical_element=optical_element,
                                               coordinates=ElementCoordinates(p=self.p,
                                                                              q=self.q,
                                                                              angle_radial=numpy.radians(self.angle_radial),
                                                                              angle_azimuthal=numpy.radians(self.angle_azimuthal)))

            #
            # this will store the propagation parameters in beamline in order to perform the propagation in the script
            #

            # 2D
            # ==
            # propagators_list = ["Fresnel",   "Fresnel (Convolution)",  "Fraunhofer",    "Integral",    "Fresnel Zoom XY"   ]
            # class_name       = ["Fresnel2D", "FresnelConvolution2D",   "Fraunhofer2D",  "Integral2D",  "FresnelZoomXY2D"   ]
            # handler_name     = ["FRESNEL_2D","FRESNEL_CONVOLUTION_2D", "FRAUNHOFER_2D", "INTEGRAL_2D", "FRESNEL_ZOOM_XY_2D"]
            if self.propagator == 0:
                propagator_info = {
                    "propagator_class_name": "Fresnel2D",
                    "propagator_handler_name": self.get_handler_name(),
                    "propagator_additional_parameters_names": [],
                    "propagator_additional_parameters_values": []}
            elif self.propagator == 1:
                propagator_info = {
                    "propagator_class_name": "FresnelConvolution2D",
                    "propagator_handler_name": self.get_handler_name(),
                    "propagator_additional_parameters_names": [],
                    "propagator_additional_parameters_values": []}
            elif self.propagator == 2:
                propagator_info = {
                    "propagator_class_name": "Fraunhofer2D",
                    "propagator_handler_name": self.get_handler_name(),
                    "propagator_additional_parameters_names": [],
                    "propagator_additional_parameters_values": []}
            elif self.propagator == 3:
                propagator_info = {
                    "propagator_class_name": "Integral2D",
                    "propagator_handler_name": self.get_handler_name(),
                    "propagator_additional_parameters_names": [],
                    "propagator_additional_parameters_values": []}
            elif self.propagator == 4:
                propagator_info = {
                    "propagator_class_name": "FresnelZoomXY2D",
                    "propagator_handler_name": self.get_handler_name(),
                    "propagator_additional_parameters_names": ['shift_half_pixel', 'magnification_x','magnification_y'],
                    "propagator_additional_parameters_values": [self.shift_half_pixel, self.magnification_x, self.magnification_y]}

            if isinstance(beamline, WOBeamline): # backcpmpatibility
                beamline.append_beamline_element(beamline_element, propagator_info)
            else:
                beamline.append_beamline_element(beamline_element)

            propagation_elements = PropagationElements()
            propagation_elements.add_beamline_element(beamline_element)

            propagation_parameters = PropagationParameters(wavefront=input_wavefront.duplicate(),
                                                           propagation_elements=propagation_elements)

            self.set_additional_parameters(propagation_parameters)

            self.setStatusMessage("Begin Propagation")

            propagator = PropagationManager.Instance()

            output_wavefront = propagator.do_propagation(propagation_parameters=propagation_parameters,
                                                         handler_name=self.get_handler_name())

            self.setStatusMessage("Propagation Completed")

            self.wavefront_to_plot = output_wavefront

            self.initializeTabs()
            self.do_plot_results()
            self.progressBarFinished()

            self.send("WofryData", WofryData(beamline=beamline, wavefront=output_wavefront))
            self.send("Trigger", TriggerIn(new_object=True))

            try:
                self.wofry_script.set_code(beamline.to_python_code())
            except:
                pass
        # except Exception as e:
        #     QMessageBox.critical(self, "Error", str(e.args[0]), QMessageBox.Ok)
        #
        #     self.setStatusMessage("")
        #     self.progressBarFinished()
        #
        #     if self.IS_DEVELOP: raise e

    def propagate_python_code(self, write_wavefront_template=True):
        txt = "#"
        txt += "\n# Import section"
        txt += "\n#"
        txt += "\nimport numpy"
        txt += "\nfrom wofry.propagator.propagator import PropagationManager, PropagationElements, PropagationParameters"
        txt += "\nfrom syned.beamline.beamline_element import BeamlineElement"
        txt += "\nfrom syned.beamline.element_coordinates import ElementCoordinates"
        txt += "\nfrom wofry.propagator.propagators2D.fresnel_zoom_xy import FresnelZoomXY2D"

        if write_wavefront_template:
            txt += "\n\n#"
            txt += "\n# create/import your input_wavefront (THIS IS A PLACEHOLDER - REPLACE WITH YOUR SOURCE)"
            txt += "\n#"
            txt += "\nfrom wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D"
            txt += "\ninput_wavefront = GenericWavefront2D.load_h5_file('wavefront2D_input.h5',filepath='wfr')"
            txt += "\n\n"

        txt += "\n\n#"
        txt += "\n# info on current oe\n#"
        txt += "\n#"
        txt_info = self.get_optical_element().info()
        lines = txt_info.split("\n")

        for line in lines:
            txt += "\n#"+line

        txt += "\n\n#"
        txt += "\n# define current oe"
        txt += "\n#"

        txt += self.get_optical_element_python_code()

        txt += "\n#"
        txt += "\n# propagating (***  ONLY THE ZOOM PROPAGATOR IS IMPLEMENTED ***)\n#"
        txt += "\n#"

        txt += "\npropagation_elements = PropagationElements()"
        txt += "\nbeamline_element = BeamlineElement(optical_element=optical_element,"
        txt += "    coordinates=ElementCoordinates(p=%f,"%(self.p)
        txt += "    q=%f,"%(self.q)
        txt += "    angle_radial=numpy.radians(%f),"%(self.angle_radial)
        txt += "    angle_azimuthal=numpy.radians(%f)))"%(self.angle_azimuthal)
        txt += "\npropagation_elements.add_beamline_element(beamline_element)"
        txt += "\npropagation_parameters = PropagationParameters(wavefront=input_wavefront.duplicate(),"
        txt += "    propagation_elements = propagation_elements)"
        txt += "\n#self.set_additional_parameters(propagation_parameters)"

        txt += "\n#"
        txt += "\npropagation_parameters.set_additional_parameters('shift_half_pixel', 1)"
        txt += "\npropagation_parameters.set_additional_parameters('magnification_x', %f)"%(self.magnification_x)
        txt += "\npropagation_parameters.set_additional_parameters('magnification_y', %f)"%(self.magnification_y)

        txt += "\n#"
        txt += "\npropagator = PropagationManager.Instance()"
        txt += "\ntry:"
        txt += "\n    propagator.add_propagator(FresnelZoomXY2D())"
        txt += "\nexcept:"
        txt += "\n    pass"
        txt += "\noutput_wavefront = propagator.do_propagation(propagation_parameters=propagation_parameters,"
        txt += "    handler_name='FRESNEL_ZOOM_XY_2D')"

        txt += "\n\nfrom srxraylib.plot.gol import plot_image"
        txt += "\nplot_image(output_wavefront.get_intensity(),output_wavefront.get_coordinate_x(), output_wavefront.get_coordinate_y(), aspect='auto')"

        return txt

    def get_handler_name(self):
        if self.propagator == 0:
            return Fresnel2D.HANDLER_NAME
        elif self.propagator == 1:
            return FresnelConvolution2D.HANDLER_NAME
        elif self.propagator == 2:
            return Fraunhofer2D.HANDLER_NAME
        elif self.propagator == 3:
            return Integral2D.HANDLER_NAME
        elif self.propagator == 4:
            return FresnelZoomXY2D.HANDLER_NAME

    def set_additional_parameters(self, propagation_parameters):
        if self.propagator <= 2:
            propagation_parameters.set_additional_parameters("shift_half_pixel", self.shift_half_pixel==1)
        elif self.propagator == 3:
            propagation_parameters.set_additional_parameters("shuffle_interval", self.shuffle_interval)
            propagation_parameters.set_additional_parameters("calculate_grid_only", self.calculate_grid_only)
        elif self.propagator == 4:
            propagation_parameters.set_additional_parameters("shift_half_pixel", self.shift_half_pixel == 1)
            propagation_parameters.set_additional_parameters("magnification_x", self.magnification_x)
            propagation_parameters.set_additional_parameters("magnification_y", self.magnification_y)

    def get_optical_element(self):
        raise NotImplementedError()

    def set_input(self, wofry_data):
        if not wofry_data is None:
            if isinstance(wofry_data, WofryData): self.input_data = wofry_data
            else: raise Exception("Only wofry_data allowed as input") # self.input_data = WofryData(wavefront=wofry_data)

            if self.is_automatic_execution:
                self.propagate_wavefront()

    def initializeTabs(self):
        size = len(self.tab)
        indexes = range(0, size)

        for index in indexes:
            self.tabs.removeTab(size-1-index)

        titles = ["Intensity","Phase"]
        self.tab = []
        self.plot_canvas = []

        for index in range(0, len(titles)):
            self.tab.append(gui.createTabPage(self.tabs, titles[index]))
            self.plot_canvas.append(None)

        for tab in self.tab:
            tab.setFixedHeight(self.IMAGE_HEIGHT)
            tab.setFixedWidth(self.IMAGE_WIDTH)

    def do_plot_results(self, progressBarValue=80):
        if not self.view_type == 0:
            if not self.wavefront_to_plot is None:

                self.progressBarSet(progressBarValue)

                titles = ["Wavefront 2D Intensity","Wavefront 2D Phase"]

                self.plot_data2D(data2D=self.wavefront_to_plot.get_intensity(),
                                 dataX=1e6*self.wavefront_to_plot.get_coordinate_x(),
                                 dataY=1e6*self.wavefront_to_plot.get_coordinate_y(),
                                 progressBarValue=progressBarValue,
                                 tabs_canvas_index=0,
                                 plot_canvas_index=0,
                                 title=titles[0],
                                 xtitle="Horizontal [$\mu$m] ( %d pixels)"%(self.wavefront_to_plot.get_coordinate_x().size),
                                 ytitle="Vertical [$\mu$m] ( %d pixels)"%(self.wavefront_to_plot.get_coordinate_y().size))

                self.plot_data2D(data2D=self.wavefront_to_plot.get_phase(from_minimum_intensity=0.1),
                             dataX=1e6*self.wavefront_to_plot.get_coordinate_x(),
                             dataY=1e6*self.wavefront_to_plot.get_coordinate_y(),
                             progressBarValue=progressBarValue,
                             tabs_canvas_index=1,
                             plot_canvas_index=1,
                             title=titles[1],
                             xtitle="Horizontal [$\mu$m] ( %d pixels)"%(self.wavefront_to_plot.get_coordinate_x().size),
                             ytitle="Vertical [$\mu$m] ( %d pixels)"%(self.wavefront_to_plot.get_coordinate_y().size))

                self.progressBarFinished()

    def receive_syned_data(self, data):
        if not data is None:
            beamline_element = data.get_beamline_element_at(-1)

            if not beamline_element is None:
                self.oe_name = beamline_element._optical_element._name
                self.p = beamline_element._coordinates._p
                self.q = beamline_element._coordinates._q
                self.angle_azimuthal = round(numpy.degrees(beamline_element._coordinates._angle_azimuthal), 6)
                self.angle_radial = round(numpy.degrees(beamline_element._coordinates._angle_radial), 6)

                self.receive_specific_syned_data(beamline_element._optical_element)
            else:
                raise Exception("Syned Data not correct: Empty Beamline Element")

    def receive_specific_syned_data(self, optical_element):
        raise NotImplementedError()

    def callResetSettings(self):
        if ConfirmDialog.confirmed(parent=self, message="Confirm Reset of the Fields?"):
            try:
                self.resetSettings()
            except:
                pass




# --------------------------------------------------------------

class OWWOOpticalElementWithBoundaryShape(OWWOOpticalElement):
    # BOUNDARY

    horizontal_shift = Setting(0.0)
    vertical_shift = Setting(0.0)

    width = Setting(0.0)
    height = Setting(0.0)

    radius = Setting(0.0)

    min_ax = Setting(0.0)
    maj_ax = Setting(0.0)

    def draw_specific_box(self):

        self.shape_box = oasysgui.widgetBox(self.tab_bas, "Boundary Shape", addSpace=True, orientation="vertical")

        gui.comboBox(self.shape_box, self, "shape", label="Boundary Shape", labelWidth=350,
                     items=["Rectangle", "Circle", "Ellipse"],
                     callback=self.set_Shape,
                     sendSelectedValue=False, orientation="horizontal")

        oasysgui.lineEdit(self.shape_box, self, "horizontal_shift", "Horizontal Shift [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.shape_box, self, "vertical_shift", "Vertical Shift [m]", labelWidth=260, valueType=float, orientation="horizontal")

        self.rectangle_box = oasysgui.widgetBox(self.shape_box, "", addSpace=False, orientation="vertical", height=60)

        oasysgui.lineEdit(self.rectangle_box, self, "width", "Width [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.rectangle_box, self, "height", "Height [m]", labelWidth=260, valueType=float, orientation="horizontal")

        self.circle_box = oasysgui.widgetBox(self.shape_box, "", addSpace=False, orientation="vertical", height=60)

        oasysgui.lineEdit(self.circle_box, self, "radius", "Radius [m]", labelWidth=260, valueType=float, orientation="horizontal")

        self.ellipse_box = oasysgui.widgetBox(self.shape_box, "", addSpace=False, orientation="vertical", height=60)

        oasysgui.lineEdit(self.ellipse_box, self, "min_ax", "Axis a [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.ellipse_box, self, "maj_ax", "Axis b [m]", labelWidth=260, valueType=float, orientation="horizontal")

        self.set_Shape()

    def set_Shape(self):
        self.rectangle_box.setVisible(self.shape == 0)
        self.circle_box.setVisible(self.shape == 1)
        self.ellipse_box.setVisible(self.shape == 2)

    def get_boundary_shape(self):
        if self.shape == 0:
            boundary_shape = Rectangle(x_left=-0.5*self.width + self.horizontal_shift,
                                       x_right=0.5*self.width + self.horizontal_shift,
                                       y_bottom=-0.5*self.height + self.vertical_shift,
                                       y_top=0.5*self.height + self.vertical_shift)

        elif self.shape == 1:
            boundary_shape = Circle( self.radius,
                                     x_center=self.horizontal_shift,
                                     y_center=self.vertical_shift)
        elif self.shape == 2:
            boundary_shape = Ellipse(min_ax_left=-0.5*self.min_ax + self.horizontal_shift,
                                     min_ax_right=0.5*self.min_ax + self.horizontal_shift,
                                     maj_ax_bottom=-0.5*self.maj_ax + self.vertical_shift,
                                     maj_ax_top=0.5*self.maj_ax + self.vertical_shift)

        return boundary_shape

    def get_boundary_shape_python_code(self):
        txt = ""
        if self.shape == 0:
            txt += "\nfrom syned.beamline.shape import Rectangle"
            txt += "\nboundary_shape = Rectangle(x_left=%g,"%(-0.5*self.width + self.horizontal_shift)
            txt += "x_right=%g,"%(0.5*self.width + self.horizontal_shift)
            txt += "y_bottom=%g,"%(-0.5*self.height + self.vertical_shift)
            txt += "y_top=%g)\n"%(0.5*self.height + self.vertical_shift)

        elif self.shape == 1:
            txt += "\nfrom syned.beamline.shape import Circle\n"
            txt += "\nboundary_shape = Circle( %g,\n"%(self.radius)
            txt += "                         x_center=%g,\n"%(self.horizontal_shift)
            txt += "                         y_center=%g)\n"%(self.vertical_shift)

        elif self.shape == 2:
            txt += "\nfrom syned.beamline.shape import Ellipse\n"
            txt += "\nboundary_shape = Ellipse(a_axis_min=%g,\n"%(-0.5*self.min_ax + self.horizontal_shift)
            txt += "                         a_axis_max=%g,\n"%(0.5*self.min_ax + self.horizontal_shift)
            txt += "                         b_axis_min=%g,\n"%(-0.5*self.maj_ax + self.vertical_shift)
            txt += "                         b_axis_max=%g)\n"%(0.5*self.maj_ax + self.vertical_shift)

        return txt



    def check_data(self):
        super().check_data()

        congruence.checkNumber(self.horizontal_shift, "Horizontal Shift")
        congruence.checkNumber(self.vertical_shift, "Vertical Shift")

        if self.shape == 0:
            congruence.checkStrictlyPositiveNumber(self.width, "Width")
            congruence.checkStrictlyPositiveNumber(self.height, "Height")
        elif self.shape == 1:
            congruence.checkStrictlyPositiveNumber(self.radius, "Radius")
        elif self.shape == 2:
            congruence.checkStrictlyPositiveNumber(self.min_ax, "(Boundary) Minor Axis")
            congruence.checkStrictlyPositiveNumber(self.maj_ax, "(Boundary) Major Axis")

    def receive_specific_syned_data(self, optical_element):
        if not optical_element is None:
            self.check_syned_instance(optical_element)

            if not optical_element._boundary_shape is None:

                left, right, bottom, top = optical_element._boundary_shape.get_boundaries()

                self.horizontal_shift = round(((right + left) / 2), 6)
                self.vertical_shift = round(((top + bottom) / 2), 6)

                if isinstance(optical_element._boundary_shape, Rectangle):
                    self.shape = 0

                    self.width = round((numpy.abs(right - left)), 6)
                    self.height = round((numpy.abs(top - bottom)), 6)

                if isinstance(optical_element._boundary_shape, Circle):
                    self.shape = 1

                if isinstance(optical_element._boundary_shape, Ellipse):
                    self.shape = 2

                    self.min_ax = round((numpy.abs(right - left)), 6)
                    self.maj_ax = round((numpy.abs(top - bottom)), 6)

                self.set_Shape()
        else:
            raise Exception("Syned Data not correct: Empty Optical Element")

# --------------------------------------------------------------

class OWWOOpticalElementWithDoubleBoundaryShape(OWWOOpticalElement):
    # BOUNDARY

    horizontal_shift = Setting(-500e-6)
    vertical_shift = Setting(-400e-6)

    width = Setting(1e-3)
    height = Setting(1e-4)

    radius = Setting(50e-6)

    min_ax = Setting(1e-3)
    maj_ax = Setting(1e-4)

    # the same for patch 2
    horizontal_shift2 = Setting(500e-6)
    vertical_shift2 = Setting(400e-6)

    width2 = Setting(1e-3)
    height2 = Setting(1e-4)

    radius2 = Setting(30e-6)

    min_ax2 = Setting(1e-3)
    maj_ax2 = Setting(1e-4)

    def draw_specific_box(self):

        self.shape_box = oasysgui.widgetBox(self.tab_bas, "Boundary Shape", addSpace=True, orientation="vertical")

        gui.comboBox(self.shape_box, self, "shape", label="Boundary Shape", labelWidth=350,
                     items=["Rectangle", "Circle", "Ellipse"],
                     callback=self.set_Shape,
                     sendSelectedValue=False, orientation="horizontal")

        oasysgui.lineEdit(self.shape_box, self, "horizontal_shift", "Horizontal Shift Patch 1[m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.shape_box, self, "vertical_shift", "Vertical Shift Patch 1 [m]", labelWidth=260, valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.shape_box, self, "horizontal_shift2", "Horizontal Shift Patch 2[m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.shape_box, self, "vertical_shift2", "Vertical Shift Patch 2[m]", labelWidth=260, valueType=float, orientation="horizontal")


        self.rectangle_box = oasysgui.widgetBox(self.shape_box, "", addSpace=False, orientation="vertical", height=120)

        oasysgui.lineEdit(self.rectangle_box, self, "width", "Width Patch 1[m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.rectangle_box, self, "height", "Height Patch 1[m]", labelWidth=260, valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.rectangle_box, self, "width2", "Width Patch 2[m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.rectangle_box, self, "height2", "Height Patch 2[m]", labelWidth=260, valueType=float, orientation="horizontal")


        self.circle_box = oasysgui.widgetBox(self.shape_box, "", addSpace=False, orientation="vertical", height=60)

        oasysgui.lineEdit(self.circle_box, self, "radius", "Radius Patch 1 [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.circle_box, self, "radius2", "Radius Patch 2 [m]", labelWidth=260, valueType=float, orientation="horizontal")

        self.ellipse_box = oasysgui.widgetBox(self.shape_box, "", addSpace=False, orientation="vertical", height=120)

        oasysgui.lineEdit(self.ellipse_box, self, "min_ax", "Axis a Patch 1 [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.ellipse_box, self, "maj_ax", "Axis b Patch 1 [m]", labelWidth=260, valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.ellipse_box, self, "min_ax2", "Axis a Patch 2 [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.ellipse_box, self, "maj_ax2", "Axis b Patch 2 [m]", labelWidth=260, valueType=float, orientation="horizontal")

        self.set_Shape()

    def set_Shape(self):
        self.rectangle_box.setVisible(self.shape == 0)
        self.circle_box.setVisible(self.shape == 1)
        self.ellipse_box.setVisible(self.shape == 2)

    def get_boundary_shape(self):
        if self.shape == 0:
            boundary_shape = DoubleRectangle(x_left1   =-0.5*self.width  + self.horizontal_shift,
                                             x_right1  = 0.5*self.width  + self.horizontal_shift,
                                             y_bottom1 =-0.5*self.height + self.vertical_shift,
                                             y_top1    = 0.5*self.height + self.vertical_shift,
                                             x_left2   =-0.5*self.width2  + self.horizontal_shift2,
                                             x_right2  = 0.5*self.width2  + self.horizontal_shift2,
                                             y_bottom2 =-0.5*self.height2 + self.vertical_shift2,
                                             y_top2    = 0.5*self.height2 + self.vertical_shift2,)

        elif self.shape == 1:
            boundary_shape = DoubleCircle( radius1=self.radius,
                                     x_center1=self.horizontal_shift,
                                     y_center1=self.vertical_shift,
                                     radius2=self.radius2,
                                     x_center2=self.horizontal_shift2,
                                     y_center2=self.vertical_shift2,
                                           )
        elif self.shape == 2:
            boundary_shape = DoubleEllipse(
                                    a_axis_min1 =-0.5*self.min_ax + self.horizontal_shift,
                                    a_axis_max1 =0.5*self.min_ax + self.horizontal_shift,
                                    b_axis_min1 =-0.5*self.maj_ax + self.vertical_shift,
                                    b_axis_max1 =0.5*self.maj_ax + self.vertical_shift,
                                    a_axis_min2 =-0.5*self.min_ax2 + self.horizontal_shift2,
                                    a_axis_max2 =0.5*self.min_ax2 + self.horizontal_shift2,
                                    b_axis_min2 =-0.5*self.maj_ax2 + self.vertical_shift2,
                                    b_axis_max2 =0.5*self.maj_ax2 + self.vertical_shift2,
                                    )

        return boundary_shape

    def get_boundary_shape_python_code(self):
        txt = ""
        if self.shape == 0:
            txt += "\nfrom syned.beamline.shape import DoubleRectangle"
            txt += "\nboundary_shape = DoubleRectangle(x_left1=%g,"%(-0.5*self.width + self.horizontal_shift)
            txt += "x_right1=%g,"%(0.5*self.width + self.horizontal_shift)
            txt += "y_bottom1=%g,"%(-0.5*self.height + self.vertical_shift)
            txt += "y_top1=%g,"%(0.5*self.height + self.vertical_shift)
            txt += "\n    x_left2=%g,"%(-0.5*self.width2 + self.horizontal_shift2)
            txt += "x_right2=%g,"%(0.5*self.width2 + self.horizontal_shift2)
            txt += "y_bottom2=%g,"%(-0.5*self.height2 + self.vertical_shift2)
            txt += "y_top2=%g)\n"%(0.5*self.height2 + self.vertical_shift2)
        elif self.shape == 1:
            txt += "\nfrom syned.beamline.shape import DoubleCircle\n"
            txt += "\nboundary_shape = DoubleCircle(radius1=%g,"%(self.radius)
            txt += "                         x_center1=%g,\n"%(self.horizontal_shift)
            txt += "                         y_center1=%g,\n"%(self.vertical_shift)
            txt += "                         radius2=%g,\n"%(self.radius2)
            txt += "                         x_center2=%g,\n"%(self.horizontal_shift2)
            txt += "                         y_center2=%g)\n"%(self.vertical_shift2)

        elif self.shape == 2:
            txt += "\nfrom syned.beamline.shape import Ellipse\n"
            txt += "\nboundary_shape = Ellipse(a_axis_min1=%g,\n"%(-0.5*self.min_ax + self.horizontal_shift)
            txt += "                         a_axis_max1=%g,\n"%(   0.5*self.min_ax + self.horizontal_shift)
            txt += "                         b_axis_min1=%g,\n"%(  -0.5*self.maj_ax + self.vertical_shift)
            txt += "                         b_axis_max1=%g)\n"%(   0.5*self.maj_ax + self.vertical_shift)
            txt += "                         a_axis_min2=%g,\n"%(  -0.5*self.min_ax2 + self.horizontal_shift2)
            txt += "                         a_axis_max2=%g,\n"%(   0.5*self.min_ax2 + self.horizontal_shift2)
            txt += "                         b_axis_min2=%g,\n"%(  -0.5*self.maj_ax2 + self.vertical_shift2)
            txt += "                         b_axis_max2=%g)\n"%(   0.5*self.maj_ax2 + self.vertical_shift2)

        return txt



    def check_data(self):
        super().check_data()

        congruence.checkNumber(self.horizontal_shift, "Horizontal Shift")
        congruence.checkNumber(self.vertical_shift, "Vertical Shift")

        if self.shape == 0:
            congruence.checkStrictlyPositiveNumber(self.width, "Width")
            congruence.checkStrictlyPositiveNumber(self.height, "Height")
        elif self.shape == 1:
            congruence.checkStrictlyPositiveNumber(self.radius, "Radius")
        elif self.shape == 2:
            congruence.checkStrictlyPositiveNumber(self.min_ax, "(Boundary) Minor Axis")
            congruence.checkStrictlyPositiveNumber(self.maj_ax, "(Boundary) Major Axis")

    def receive_specific_syned_data(self, optical_element):
        if not optical_element is None:
            self.check_syned_instance(optical_element)

            if not optical_element._boundary_shape is None:

                left, right, bottom, top = optical_element._boundary_shape.get_boundaries()

                self.horizontal_shift = round(((right + left) / 2), 6)
                self.vertical_shift = round(((top + bottom) / 2), 6)

                if isinstance(optical_element._boundary_shape, Rectangle):
                    self.shape = 0

                    self.width = round((numpy.abs(right - left)), 6)
                    self.height = round((numpy.abs(top - bottom)), 6)

                if isinstance(optical_element._boundary_shape, Circle):
                    self.shape = 1

                if isinstance(optical_element._boundary_shape, Ellipse):
                    self.shape = 2

                    self.min_ax = round((numpy.abs(right - left)), 6)
                    self.maj_ax = round((numpy.abs(top - bottom)), 6)

                self.set_Shape()
        else:
            raise Exception("Syned Data not correct: Empty Optical Element")

# --------------------------------------------------------------

class OWWOOpticalElementWithSurfaceShape(OWWOOpticalElementWithBoundaryShape):

    # SURFACE

    convexity = Setting(0)
    is_cylinder = Setting(1)
    cylinder_direction = Setting(0)

    p_surface = Setting(0.0)
    q_surface = Setting(0.0)

    calculate_sphere_parameter = Setting(0)
    calculate_ellipsoid_parameter = Setting(0)
    calculate_paraboloid_parameter = Setting(0)
    calculate_hyperboloid_parameter = Setting(0)
    calculate_torus_parameter = Setting(0)


    # SPHERE
    radius_surface = Setting(0.0)

    # ELLIPSOID/HYPERBOLOID
    min_ax_surface = Setting(0.0)
    maj_ax_surface = Setting(0.0)

    # PARABOLOID
    parabola_parameter_surface = Setting(0.0)
    at_infinty_surface = Setting(0.0)

    # TORUS
    min_radius_surface = Setting(0.0)
    maj_radius_surface = Setting(0.0)

    def draw_specific_box(self, tab_oe):

        super().draw_specific_box()

        self.surface_shape_box = oasysgui.widgetBox(tab_oe, "Surface Shape", addSpace=True, orientation="vertical", height=190)

        gui.comboBox(self.surface_shape_box, self, "surface_shape", label="Surface Shape", labelWidth=350,
                     items=["Plane", "Sphere", "Ellipsoid", "Paraboloid", "Hyperboloid", "Toroidal"],
                     callback=self.set_SurfaceParameters,
                     sendSelectedValue=False, orientation="horizontal")


        self.plane_box = oasysgui.widgetBox(self.surface_shape_box, "", addSpace=False, orientation="vertical", height=90)

        self.sphere_box = oasysgui.widgetBox(self.surface_shape_box, "", addSpace=False, orientation="vertical", height=90)
        self.ellipsoid_box = oasysgui.widgetBox(self.surface_shape_box, "", addSpace=False, orientation="vertical", height=90)
        self.paraboloid_box = oasysgui.widgetBox(self.surface_shape_box, "", addSpace=False, orientation="vertical", height=115)
        self.hyperboloid_box = oasysgui.widgetBox(self.surface_shape_box, "", addSpace=False, orientation="vertical", height=90)
        self.torus_box = oasysgui.widgetBox(self.surface_shape_box, "", addSpace=False, orientation="vertical", height=90)

        # SPHERE --------------------------

        gui.comboBox(self.sphere_box, self, "calculate_sphere_parameter", label="Sphere Shape", labelWidth=350,
                     items=["Manual", "Automatic"],
                     callback=self.set_SphereShape,
                     sendSelectedValue=False, orientation="horizontal")

        self.sphere_box_1 = oasysgui.widgetBox(self.sphere_box, "", addSpace=False, orientation="vertical", height=60)
        self.sphere_box_2 = oasysgui.widgetBox(self.sphere_box, "", addSpace=False, orientation="vertical", height=60)

        oasysgui.lineEdit(self.sphere_box_1, self, "radius_surface", "Radius [m]", labelWidth=260, valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.sphere_box_2, self, "p_surface", "First Focus to O.E. Center (P) [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.sphere_box_2, self, "q_surface", "O.E. Center to Second Focus (Q) [m]", labelWidth=260, valueType=float, orientation="horizontal")


        # ELLIPSOID --------------------------

        gui.comboBox(self.ellipsoid_box, self, "calculate_ellipsoid_parameter", label="Ellipsoid Shape", labelWidth=350,
                     items=["Manual", "Automatic"],
                     callback=self.set_EllipsoidShape,
                     sendSelectedValue=False, orientation="horizontal")

        self.ellipsoid_box_1 = oasysgui.widgetBox(self.ellipsoid_box, "", addSpace=False, orientation="vertical", height=60)
        self.ellipsoid_box_2 = oasysgui.widgetBox(self.ellipsoid_box, "", addSpace=False, orientation="vertical", height=60)

        oasysgui.lineEdit(self.ellipsoid_box_1, self, "min_ax_surface", "Minor Axis [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.ellipsoid_box_1, self, "maj_ax_surface", "Major Axis [m]", labelWidth=260, valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.ellipsoid_box_2, self, "p_surface", "First Focus to O.E. Center (P) [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.ellipsoid_box_2, self, "q_surface", "O.E. Center to Second Focus (Q) [m]", labelWidth=260, valueType=float, orientation="horizontal")


        # PARABOLOID --------------------------

        gui.comboBox(self.paraboloid_box, self, "calculate_paraboloid_parameter", label="Sphere Shape", labelWidth=350,
                     items=["Manual", "Automatic"],
                     callback=self.set_ParaboloidShape,
                     sendSelectedValue=False, orientation="horizontal")

        self.paraboloid_box_1 = oasysgui.widgetBox(self.paraboloid_box, "", addSpace=False, orientation="vertical", height=85)
        self.paraboloid_box_2 = oasysgui.widgetBox(self.paraboloid_box, "", addSpace=False, orientation="vertical", height=85)

        oasysgui.lineEdit(self.paraboloid_box_1, self, "parabola_parameter_surface", "Parabola Parameter [m]", labelWidth=260, valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.paraboloid_box_2, self, "p_surface", "First Focus to O.E. Center (P) [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.paraboloid_box_2, self, "q_surface", "O.E. Center to Second Focus (Q) [m]", labelWidth=260, valueType=float, orientation="horizontal")

        gui.comboBox(self.paraboloid_box_2, self, "at_infinty_surface", label="At infinity", labelWidth=350,
                     items=["Source", "Image"],
                     sendSelectedValue=False, orientation="horizontal")


        # HYPERBOLOID --------------------------

        gui.comboBox(self.hyperboloid_box, self, "calculate_hyperboloid_parameter", label="Hyperboloid Shape", labelWidth=350,
                     items=["Manual", "Automatic"],
                     callback=self.set_HyperboloidShape,
                     sendSelectedValue=False, orientation="horizontal")

        self.hyperboloid_box_1 = oasysgui.widgetBox(self.hyperboloid_box, "", addSpace=False, orientation="vertical", height=60)
        self.hyperboloid_box_2 = oasysgui.widgetBox(self.hyperboloid_box, "", addSpace=False, orientation="vertical", height=60)

        oasysgui.lineEdit(self.hyperboloid_box_1, self, "min_ax_surface", "Minor Axis [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.hyperboloid_box_1, self, "maj_ax_surface", "Major Axis [m]", labelWidth=260, valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.hyperboloid_box_2, self, "p_surface", "First Focus to O.E. Center (P) [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.hyperboloid_box_2, self, "q_surface", "O.E. Center to Second Focus (Q) [m]", labelWidth=260, valueType=float, orientation="horizontal")


        # TORUS --------------------------

        gui.comboBox(self.torus_box, self, "calculate_torus_parameter", label="Torus Shape", labelWidth=350,
                     items=["Manual", "Automatic"],
                     callback=self.set_TorusShape,
                     sendSelectedValue=False, orientation="horizontal")

        self.torus_box_1 = oasysgui.widgetBox(self.torus_box, "", addSpace=False, orientation="vertical", height=60)
        self.torus_box_2 = oasysgui.widgetBox(self.torus_box, "", addSpace=False, orientation="vertical", height=60)

        oasysgui.lineEdit(self.torus_box_1, self, "min_radius_surface", "Minor radius [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.torus_box_1, self, "maj_radius_surface", "Major radius [m]", labelWidth=260, valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.torus_box_2, self, "p_surface", "First Focus to O.E. Center (P) [m]", labelWidth=260, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.torus_box_2, self, "q_surface", "O.E. Center to Second Focus (Q) [m]", labelWidth=260, valueType=float, orientation="horizontal")

        # -----------------------------------------------------
        # -----------------------------------------------------

        self.surface_orientation_box = oasysgui.widgetBox(tab_oe, "Surface Orientation", addSpace=False, orientation="vertical")

        gui.comboBox(self.surface_orientation_box, self, "convexity", label="Convexity", labelWidth=350,
                     items=["Upward", "Downward"],
                     sendSelectedValue=False, orientation="horizontal")

        gui.comboBox(self.surface_orientation_box, self, "is_cylinder", label="Cylinder", labelWidth=350,
                     items=["No", "Yes"], callback=self.set_Cylinder,
                     sendSelectedValue=False, orientation="horizontal")

        self.cylinder_box_1 = oasysgui.widgetBox(self.surface_orientation_box, "", addSpace=False, orientation="vertical", height=25)
        self.cylinder_box_2 = oasysgui.widgetBox(self.surface_orientation_box, "", addSpace=False, orientation="vertical", height=25)

        gui.comboBox(self.cylinder_box_1, self, "cylinder_direction", label="Cylinder Direction", labelWidth=350,
                     items=["Tangential", "Sagittal"],
                     sendSelectedValue=False, orientation="horizontal")

        self.set_SurfaceParameters()

    def set_SphereShape(self):
        self.sphere_box_1.setVisible(self.calculate_sphere_parameter==0)
        self.sphere_box_2.setVisible(self.calculate_sphere_parameter==1)

    def set_EllipsoidShape(self):
        self.ellipsoid_box_1.setVisible(self.calculate_ellipsoid_parameter==0)
        self.ellipsoid_box_2.setVisible(self.calculate_ellipsoid_parameter==1)

    def set_ParaboloidShape(self):
        self.paraboloid_box_1.setVisible(self.calculate_paraboloid_parameter==0)
        self.paraboloid_box_2.setVisible(self.calculate_paraboloid_parameter==1)

    def set_HyperboloidShape(self):
        self.hyperboloid_box_1.setVisible(self.calculate_hyperboloid_parameter==0)
        self.hyperboloid_box_2.setVisible(self.calculate_hyperboloid_parameter==1)

    def set_TorusShape(self):
        self.torus_box_1.setVisible(self.calculate_torus_parameter==0)
        self.torus_box_2.setVisible(self.calculate_torus_parameter==1)


    def set_Cylinder(self):
        self.cylinder_box_1.setVisible(self.is_cylinder==1)
        self.cylinder_box_2.setVisible(self.is_cylinder==0)

    def set_SurfaceParameters(self):
        self.plane_box.setVisible(self.surface_shape == 0)

        if self.surface_shape == 1 :
            self.sphere_box.setVisible(True)
            self.set_SphereShape()
        else:
            self.sphere_box.setVisible(False)

        if self.surface_shape == 2 :
            self.ellipsoid_box.setVisible(True)
            self.set_EllipsoidShape()
        else:
            self.ellipsoid_box.setVisible(False)

        if self.surface_shape == 3 :
            self.paraboloid_box.setVisible(True)
            self.set_ParaboloidShape()
        else:
            self.paraboloid_box.setVisible(False)

        if self.surface_shape == 4 :
            self.hyperboloid_box.setVisible(True)
            self.set_HyperboloidShape()
        else:
            self.hyperboloid_box.setVisible(False)

        if self.surface_shape == 5 :
            self.torus_box.setVisible(True)
            self.set_TorusShape()
        else:
            self.torus_box.setVisible(False)

        if self.surface_shape in (1,2,3,4):
            self.surface_orientation_box.setVisible(True)
            self.set_Cylinder()
        else:
            self.surface_orientation_box.setVisible(False)

    def get_surface_shape(self):
        if self.surface_shape == 0:
            surface_shape = Plane()

        # SPHERE --------------------------
        elif self.surface_shape == 1:
            if self.calculate_sphere_parameter == 0:
                if self.is_cylinder == 0:
                    surface_shape = Sphere(radius=self.radius_surface,
                                           convexity=self.convexity)
                else:
                    surface_shape = SphericalCylinder(radius=self.radius_surface,
                                                      convexity=self.convexity,
                                                      cylinder_direction=self.cylinder_direction)
            elif self.calculate_sphere_parameter == 1:
                if self.is_cylinder == 0:
                    surface_shape = Sphere(convexity=self.convexity)
                else:
                    surface_shape = SphericalCylinder(convexity=self.convexity,
                                                      cylinder_direction=self.cylinder_direction)

                surface_shape.initialize_from_p_q(self.p_surface, self.q_surface, numpy.radians(90-self.angle_radial))

                self.radius_surface = round(surface_shape.get_radius(), 6)

        # ELLIPSOID --------------------------
        elif self.surface_shape == 2:
            if self.calculate_ellipsoid_parameter == 0:
                if self.is_cylinder == 0:
                    surface_shape = Ellipsoid(min_axis=self.min_ax_surface,
                                              maj_axis=self.maj_ax_surface,
                                              convexity=self.convexity)
                else:
                    surface_shape = EllipticalCylinder(min_axis=self.min_ax_surface,
                                                       maj_axis=self.maj_ax_surface,
                                                       convexity=self.convexity,
                                                       cylinder_direction=self.cylinder_direction)
            elif self.calculate_ellipsoid_parameter == 1:
                if self.is_cylinder == 0:
                    surface_shape = Ellipsoid(convexity=self.convexity)
                else:
                    surface_shape = EllipticalCylinder(convexity=self.convexity,
                                                       cylinder_direction=self.cylinder_direction)

                surface_shape.initialize_from_p_q(self.p_surface, self.q_surface, numpy.radians(90-self.angle_radial))

                self.min_ax_surface = round(surface_shape._min_axis, 6)
                self.maj_ax_surface = round(surface_shape._maj_axis, 6)

        # PARABOLOID --------------------------
        elif self.surface_shape == 3:
            if self.calculate_paraboloid_parameter == 0:
                if self.is_cylinder == 0:
                    surface_shape = Paraboloid(parabola_parameter=self.parabola_parameter_surface,
                                               convexity=self.convexity)
                else:
                    surface_shape = ParabolicCylinder(parabola_parameter=self.parabola_parameter_surface,
                                                      convexity=self.convexity,
                                                      cylinder_direction=self.cylinder_direction)
            elif self.calculate_paraboloid_parameter == 1:
                if self.is_cylinder == 0:
                    surface_shape = Paraboloid(convexity=self.convexity)
                else:
                    surface_shape = ParabolicCylinder(convexity=self.convexity,
                                                    cylinder_direction=self.cylinder_direction)

                surface_shape.initialize_from_p_q(self.p_surface, self.q_surface, numpy.radians(90-self.angle_radial), at_infinity=self.at_infinty_surface)

                self.parabola_parameter_surface = round(surface_shape._parabola_parameter, 6)

        # HYPERBOLOID --------------------------
        elif self.surface_shape == 4:
            if self.calculate_hyperboloid_parameter == 0:
                if self.is_cylinder == 0:
                    surface_shape = Hyperboloid(min_axis=self.min_ax_surface,
                                                maj_axis=self.maj_ax_surface,
                                                convexity=self.convexity)
                else:
                    surface_shape = HyperbolicCylinder(min_axis=self.min_ax_surface,
                                                       maj_axis=self.maj_ax_surface,
                                                       convexity=self.convexity,
                                                       cylinder_direction=self.cylinder_direction)
            elif self.calculate_ellipsoid_parameter == 1:
                raise NotImplementedError("HYPERBOLOID, you should not be here!")

        # TORUS --------------------------
        elif self.surface_shape == 5:
            if self.calculate_torus_parameter == 0:
                surface_shape = Toroidal(min_radius=self.min_radius_surface,
                                      maj_radius=self.maj_radius_surface)
            elif self.calculate_torus_parameter == 1:
                surface_shape = Toroidal()

                surface_shape.initialize_from_p_q(self.p_surface, self.q_surface, numpy.radians(90-self.angle_radial))

                self.min_radius_surface = round(surface_shape._min_radius, 6)
                self.maj_radius_surface = round(surface_shape._maj_radius, 6)

        return surface_shape

    def check_data(self):
        super().check_data()

        if self.surface_shape == 1: # SPHERE
            if self.calculate_sphere_parameter == 0:
                congruence.checkStrictlyPositiveNumber(self.radius_surface, "(Surface) Radius")
            elif self.calculate_sphere_parameter == 1:
                congruence.checkStrictlyPositiveNumber(self.p_surface, "(Surface) P")

        if self.surface_shape == 2: # ELLIPSOID
            if self.calculate_ellipsoid_parameter == 0:
                congruence.checkStrictlyPositiveNumber(self.min_ax_surface, "(Surface) Minor Axis")
                congruence.checkStrictlyPositiveNumber(self.maj_ax_surface, "(Surface) Major Axis")
            elif self.calculate_ellipsoid_parameter == 1:
                congruence.checkStrictlyPositiveNumber(self.p_surface, "(Surface) P")
                congruence.checkStrictlyPositiveNumber(self.q_surface, "(Surface) Q")

                if self.is_cylinder and self.cylinder_direction == Direction.SAGITTAL:
                    raise NotImplementedError("Sagittal automatic calculation is not supported, yet")

        if self.surface_shape == 3: # PARABOLOID
            if self.calculate_paraboloid_parameter == 0:
                congruence.checkStrictlyPositiveNumber(self.parabola_parameter_surface, "(Surface) Parabola Parameter")
            elif self.calculate_paraboloid_parameter == 1:
                congruence.checkStrictlyPositiveNumber(self.p_surface, "(Surface) P")
                congruence.checkStrictlyPositiveNumber(self.q_surface, "(Surface) Q")

                if self.is_cylinder and self.cylinder_direction == Direction.SAGITTAL:
                    raise NotImplementedError("Sagittal automatic calculation is not supported, yet")

        if self.surface_shape == 4: # HYPERBOLOID
            if self.calculate_hyperboloid_parameter == 0:
                congruence.checkStrictlyPositiveNumber(self.min_ax_surface, "(Surface) Minor Axis")
                congruence.checkStrictlyPositiveNumber(self.maj_ax_surface, "(Surface) Major Axis")
            elif self.calculate_hyperboloid_parameter == 1:
                raise NotImplementedError("Automatic calculation is not supported, yet")

        if self.surface_shape == 5: # TORUS
            if self.calculate_torus_parameter == 0:
                congruence.checkStrictlyPositiveNumber(self.min_radius_surface, "(Surface) Minor Radius")
                congruence.checkStrictlyPositiveNumber(self.maj_radius_surface, "(Surface) Major Radius")
            elif self.calculate_torus_parameter == 1:
                congruence.checkStrictlyPositiveNumber(self.p_surface, "(Surface) P")
                congruence.checkStrictlyPositiveNumber(self.q_surface, "(Surface) Q")

    def receive_specific_syned_data(self, optical_element):
        super().receive_specific_syned_data(optical_element)

        #TODO: check and passage of shapes

        raise NotImplementedError()




#from syned.beamline.optical_element_with_two_surface_shapes import OpticalElementsWithTwoSurfaceShapes
from orangecontrib.esrf.syned.util.optical_element_with_two_surface_shapes import OpticalElementsWithTwoSurfaceShapes
from syned.beamline.shape import Plane

class Lens(OpticalElementsWithTwoSurfaceShapes):
    def __init__(self,
                 name="Undefined",
                 surface_shape1=None,
                 surface_shape2=None,
                 boundary_shape=None,
                 material="",
                 thickness=0.0):

        if surface_shape1 is None: surface_shape1 = Plane()
        if surface_shape2 is None: surface_shape2 = Plane()

        super().__init__(name=name, surface_shape1=surface_shape1, surface_shape2=surface_shape2,
                         boundary_shape=boundary_shape)
        self._material = material
        self._thickness = thickness
        # support text contaning name of variable, help text and unit. Will be stored in self._support_dictionary
        self._set_support_text([
                    ("name",                "Name" ,                                "" ),
                    ("surface_shape1",      "Surface shape 1",                      "" ),
                    ("surface_shape2",      "Surface shape 2",                      ""),
                    ("boundary_shape",      "Boundary shape",                       "" ),
                    ("material",            "Material (element, compound or name)", "" ),
                    ("thickness",           "Thickness",                            "m"),
            ] )

    def get_thickness(self):
        return self._thickness

    def get_material(self):
        return self._material

    def get_boundary_shape(self):
        return self._boundary_shape

    def get_surface_shape1(self):
        return self.get_surface_shape(index=1)

    def get_surface_shape2(self):
        return self.get_surface_shape(index=2)



import numpy
from PyQt5.QtCore import QRect
from PyQt5.QtWidgets import QApplication, QMessageBox

from orangewidget import gui
from orangewidget.settings import Setting

from oasys.widgets.widget import OWWidget
from oasys.widgets import gui as oasysgui

from oasys.util.oasys_objects import OasysPreProcessorData, OasysErrorProfileData, OasysSurfaceData

import oasys.util.oasys_util as OU

from silx.gui.plot import Plot2D
from PyQt5 import QtGui

from scipy import interpolate

class OWSurfaceFileReader(OWWidget):
    name = "ESRF Surface File Merger"  # TODO: remove ESRF when replacing oasys merger
    id = "surface_file_merger"
    description = "Surface File Merger"
    icon = "icons/surface_merger.png"
    author = "Luca Rebuffi and Manuel Sanchez del Rio"
    maintainer_email = "lrebuffi@anl.gov"
    priority = 5
    category = ""
    keywords = ["surface_file_mberger"]

    inputs = [("Surface Data #1", object, "set_input_1"),
              ("Surface Data #2", object, "set_input_2"),
              ]

    outputs = [{"name": "PreProcessor_Data",
                "type": OasysPreProcessorData,
                "doc": "PreProcessor Data",
                "id": "PreProcessor_Data"},
               {"name": "Surface_Data",
                "type": OasysSurfaceData,
                "doc": "Surface Data",
                "id": "Surface_Data"}]


    want_main_area = 1
    want_control_area = 1

    MAX_WIDTH = 1320
    MAX_HEIGHT = 700

    IMAGE_WIDTH = 860
    IMAGE_HEIGHT = 645

    CONTROL_AREA_WIDTH = 405
    TABS_AREA_HEIGHT = 618

    xx = None
    yy = None
    zz = None

    xx_1 = None
    yy_1 = None
    zz_1 = None

    xx_2 = None
    yy_2 = None
    zz_2 = None

    surface_data = None
    preprocessor_data = None


    surface_file_name = Setting('merged_surface.hdf5')
    unmatched_surfaces = Setting(1)
    interpolated_length_x = Setting(0.1)
    interpolated_length_y = Setting(1.0)
    interpolated_nx = Setting(500)
    interpolated_ny = Setting(50)
    factor_surface1 = Setting(1.0)
    factor_surface2 = Setting(1.0)

    def __init__(self):
        super().__init__()

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width() * 0.05),
                               round(geom.height() * 0.05),
                               round(min(geom.width() * 0.98, self.MAX_WIDTH)),
                               round(min(geom.height() * 0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        gui.separator(self.controlArea)

        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Merge", callback=self.compute)

        button.setFixedHeight(45)

        input_box = oasysgui.widgetBox(self.controlArea, "Input", addSpace=True, orientation="vertical",
                                         height=self.TABS_AREA_HEIGHT)


        #
        oasysgui.lineEdit(input_box, self, "factor_surface1",
                     label="Factor for surface1", addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)

        oasysgui.lineEdit(input_box, self, "factor_surface2",
                     label="Factor for surface2", addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)


        #
        gui.comboBox(input_box, self, "unmatched_surfaces", label="if unmatched surfaces", labelWidth=250,
                                     items=["Do nothing (error)", "Use intersection grid", "Customized grid"],
                                     sendSelectedValue=False, orientation="horizontal",
                                     callback=self.set_visible)



        self.interpoleted_id = oasysgui.widgetBox(input_box, "", addSpace=True, orientation="vertical")

        oasysgui.lineEdit(self.interpoleted_id, self, "interpolated_length_x",
                     label="Interpolated width (X)", addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)

        oasysgui.lineEdit(self.interpoleted_id, self, "interpolated_length_y",
                     label="Interpolated length (Y)", addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)

        oasysgui.lineEdit(self.interpoleted_id, self, "interpolated_nx",
                     label="Interpolated points along with (X)", addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)

        oasysgui.lineEdit(self.interpoleted_id, self, "interpolated_ny",
                     label="Interpolated points along with (Y)", addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)


        # file
        input_box_l = oasysgui.widgetBox(input_box, "", addSpace=True, orientation="horizontal", height=self.TABS_AREA_HEIGHT)

        self.le_surface_file_name = oasysgui.lineEdit(input_box_l, self, "surface_file_name", "Surface File Name",
                                                        labelWidth=120, valueType=str, orientation="horizontal")

        gui.button(input_box_l, self, "...", callback=self.selectSurfaceFile)



        #
        # tabs results
        #
        tabs_setting = oasysgui.tabWidget(self.mainArea)

        self.tab_surface1 = oasysgui.createTabPage(tabs_setting, "Surface input 1")
        self.tab_surface2 = oasysgui.createTabPage(tabs_setting, "Surface input 2")
        self.tab_surfaceR = oasysgui.createTabPage(tabs_setting, "Surface Result")
        tab_output = oasysgui.createTabPage(tabs_setting, "Output")

        self.info_id = oasysgui.textArea(height=self.IMAGE_HEIGHT - 35)
        info_box = oasysgui.widgetBox(tab_output, "", addSpace=True, orientation="horizontal",
                                      height=self.IMAGE_HEIGHT - 20, width=self.IMAGE_WIDTH - 20)
        info_box.layout().addWidget(self.info_id)

        gui.rubber(self.mainArea)

        self.set_visible()

    def clean_widgets(self, surface_index):
        if surface_index == 1:
            layout = self.tab_surface1.layout()
        else:
            layout = self.tab_surface2.layout()
        for i in reversed(range(layout.count())):
            layout.itemAt(i).widget().setParent(None)

        layout = self.tab_surfaceR.layout()
        for i in reversed(range(layout.count())):
            layout.itemAt(i).widget().setParent(None)

        self.writeStdOut("", initialize=True)


    def set_input_1(self, data):
        self.writeStdOut("\nCalled set_input_1 with " + str(data))
        if data is None:
            self.xx_1 = None
            self.yy_1 = None
            self.zz_1 = None
            self.clean_widgets(1)
        else:
            if isinstance(data, OasysPreProcessorData):
                self.xx_1 = data.error_profile_data.surface_data.xx
                self.yy_1 = data.error_profile_data.surface_data.yy
                self.zz_1 = data.error_profile_data.surface_data.zz
                self.plot_surface1()
            elif isinstance(data, OasysSurfaceData):
                self.xx_1 = data.xx
                self.yy_1 = data.yy
                self.zz_1 = data.zz
                self.plot_surface1()
            else:
                QMessageBox.critical(self, "Error",
                                     "Data Type #1 not recognized",
                                     QMessageBox.Ok)


    def set_input_2(self, data):
        self.writeStdOut("\nCalled set_input_2 with " + str(data))
        if data is None:
            self.xx_2 = None
            self.yy_2 = None
            self.zz_2 = None
            self.clean_widgets(2)

        else:
            if isinstance(data, OasysPreProcessorData):
                self.xx_2 = data.error_profile_data.surface_data.xx
                self.yy_2 = data.error_profile_data.surface_data.yy
                self.zz_2 = data.error_profile_data.surface_data.zz
                self.plot_surface2()
            elif isinstance(data, OasysSurfaceData):
                self.xx_2 = data.xx
                self.yy_2 = data.yy
                self.zz_2 = data.zz
                self.plot_surface2()
            else:
                QMessageBox.critical(self, "Error",
                                     "Data Type #2 not recognized",
                                     QMessageBox.Ok)

    def set_visible(self):
        if self.unmatched_surfaces == 2:
            self.interpoleted_id.setVisible(True)
        else:
            self.interpoleted_id.setVisible(False)

    def writeStdOut(self, text="", initialize=False):
        cursor = self.info_id.textCursor()
        if initialize:
            self.info_id.setText(text)
        else:
            cursor.movePosition(QtGui.QTextCursor.End)
            cursor.insertText(text)

    def plot_data2D(self, data2D, dataX, dataY, tab_canvas, title="", xtitle=None, ytitle=None, vmax=None):

        layout = tab_canvas.layout()
        for i in reversed(range(layout.count())):
            layout.itemAt(i).widget().setParent(None)

        if xtitle is None:
            xtitle = "X (%d pixels)" % dataX.size
        if ytitle is None:
            ytitle = "Y (%d pixels)" % dataY.size

        origin = (dataX[0], dataY[0])
        scale = (dataX[1] - dataX[0], dataY[1] - dataY[0])

        if vmax is None:
            colormap = {"name": "temperature", "normalization": "linear",
                        "autoscale": True, "vmin": 0, "vmax": 0, "colors": 256}
        else:
            colormap = {"name": "temperature", "normalization": "linear",
                        "autoscale": False, "vmin": 0, "vmax": vmax, "colors": 256}

        tmp = Plot2D()
        tmp.resetZoom()
        tmp.setXAxisAutoScale(True)
        tmp.setYAxisAutoScale(True)
        tmp.setGraphGrid(False)
        tmp.setKeepDataAspectRatio(True)
        tmp.yAxisInvertedAction.setVisible(False)
        tmp.setXAxisLogarithmic(False)
        tmp.setYAxisLogarithmic(False)
        tmp.getMaskAction().setVisible(False)
        tmp.getRoiAction().setVisible(False)
        tmp.getColormapAction().setVisible(True)
        tmp.setKeepDataAspectRatio(False)
        tmp.addImage(data2D,legend="1",scale=scale,origin=origin,colormap=colormap,replace=True)
        tmp.setActiveImage("1")
        tmp.setGraphXLabel(xtitle)
        tmp.setGraphYLabel(ytitle)
        tmp.setGraphTitle(title)

        layout.addWidget(tmp)

    def plot_surface1(self, vmax=None):
        self.plot_data2D(self.zz_1 * self.factor_surface1, self.xx_1, self.yy_1, self.tab_surface1,
                         vmax=vmax, title="input surface 1 * %g" % self.factor_surface1)

    def plot_surface2(self, vmax=None):
        self.plot_data2D(self.zz_2 * self.factor_surface2, self.xx_2, self.yy_2, self.tab_surface2,
                         vmax=vmax, title="input surface 2 * %g" % self.factor_surface2)


    def compute(self, plot_data=True):
        try:
            if not self.xx_1 is None and not self.xx_2 is None:
                xx_1 = self.xx_1
                yy_1 = self.yy_1
                zz_1 = self.zz_1

                xx_2 = self.xx_2
                yy_2 = self.yy_2
                zz_2 = self.zz_2

                self.writeStdOut("",initialize=True)
                self.writeStdOut("Surface 1 dimensions: X: %f (%d pixels), Y: %f (%d pixels)\n" %
                                 (numpy.abs(xx_1[-1]-xx_1[0]), xx_1.size, numpy.abs(yy_1[-1]-yy_1[0]), yy_1.size,), initialize=False)
                self.writeStdOut("Surface 2 dimensions: X: %f (%d pixels), Y: %f (%d pixels)\n" %
                                 (numpy.abs(xx_2[-1]-xx_2[0]), xx_2.size, numpy.abs(yy_2[-1]-yy_2[0]), yy_2.size,), initialize=False)

                if numpy.array_equal(xx_1,xx_2) and numpy.array_equal(yy_1,yy_2):
                    self.writeStdOut("Surface 1 and 2 grids do match\n", initialize=False)
                    match = 1
                else:
                    self.writeStdOut("Surface 1 and 2 grids DO NOT match\n", initialize=False)
                    match = 0


                if match:
                    xx = xx_2.copy()
                    yy = yy_2.copy()
                    zz = zz_2 * self.factor_surface2 + zz_1 * self.factor_surface1
                else:
                    if self.unmatched_surfaces == 0:
                        self.writeStdOut("Aborted \n", initialize=False)
                        raise ValueError("The two surfaces cannot be merged: dimensions or binning incompatible")
                    else:
                        if self.unmatched_surfaces == 1:
                            xx_min = numpy.max((xx_1[0], xx_2[0]))
                            xx_max = numpy.min((xx_1[-1], xx_2[-1]))
                            xx_n = numpy.min((xx_1.size, xx_2.size))
                            yy_min = numpy.max((yy_1[0], yy_2[0]))
                            yy_max = numpy.min((yy_1[-1], yy_2[-1]))
                            yy_n = numpy.min((yy_1.size, yy_2.size))
                            xx = numpy.linspace(xx_min, xx_max, xx_n)
                            yy = numpy.linspace(yy_min, yy_max, yy_n)
                        elif self.unmatched_surfaces == 2:
                            xx = numpy.linspace(-0.5 * self.interpolated_length_x,
                                                0.5 * self.interpolated_length_x,
                                                int(self.interpolated_nx))
                            yy = numpy.linspace(-0.5 * self.interpolated_length_y,
                                                0.5 * self.interpolated_length_y,
                                                int(self.interpolated_ny))


                        XX = numpy.outer(xx, numpy.ones_like(yy))
                        YY = numpy.outer(numpy.ones_like(xx), yy)
                        XX_1 = numpy.outer(xx_1, numpy.ones_like(yy_1))
                        YY_1 = numpy.outer(numpy.ones_like(xx_1), yy_1)
                        XX_2 = numpy.outer(xx_2, numpy.ones_like(yy_2))
                        YY_2 = numpy.outer(numpy.ones_like(xx_2), yy_2)
                        print(XX_1.shape, YY_1.shape, zz_1.shape, XX_2.shape, YY_2.shape, zz_2.shape, XX.shape, YY.shape)
                        zz_1i = interpolate.griddata(
                            (XX_1.flatten(), YY_1.flatten()),
                            (zz_1.T).flatten(),
                            (XX, YY), method='cubic', fill_value=0.0, rescale=True)
                        zz_2i = interpolate.griddata(
                            (XX_2.flatten(), YY_2.flatten()),
                            (zz_2.T).flatten(),
                            (XX, YY), method='cubic', fill_value=0.0, rescale=True)
                        zz = zz_1i.T * self.factor_surface1 + zz_2i.T * self.factor_surface2
                        print(zz.shape)

                self.writeStdOut("Result Surface dimensions: X: %f (%d pixels), Y: %f (%d pixels)\n" %
                                         (numpy.abs(xx[-1] - xx[0]), xx.size, numpy.abs(yy[-1] - yy[0]), yy.size,),
                                         initialize=False)

                self.writeStdOut("Surface 1\n   X from %f to %f\n   Y from %f to %f\n   Zmax %g\n   Zmax * factor: %g\n" %
                                         (xx_1[0], xx_1[-1], yy_1[0], yy_1[-1], zz_1.max(), zz_1.max() * self.factor_surface1),
                                         initialize=False)
                if match == 0:
                    self.writeStdOut("Surface 1 interpolated\n   X from %f to %f\n   Y from %f to %f\n   Zmax %g\n   Zmax * factor: %g\n" %
                                             (xx[0], xx[-1], yy[0], yy[-1], zz_1i.max(), zz_1i.max() * self.factor_surface1),
                                             initialize=False)
                self.writeStdOut("Surface 2\n   X from %f to %f\n   Y from %f to %f\n   Zmax %g\n   Zmax * factor: %g\n" %
                                         (xx_2[0], xx_2[-1], yy_2[0], yy_2[-1], zz_2.max(), zz_2.max() * self.factor_surface2),
                                         initialize=False)
                if match == 0:
                    self.writeStdOut("Surface 2 interpolated\n   X from %f to %f\n   Y from %f to %f\n   Zmax %g\n   Zmax * factor: %g\n" %
                                             (xx[0], xx[-1], yy[0], yy[-1], zz_2i.max(), zz_2i.max() * self.factor_surface2),
                                             initialize=False)
                self.writeStdOut("Surface Result\n   X from %f to %f\n   Y from %f to %f\n   Zmax: %g\n" %
                                         (xx[0], xx[-1], yy[0], yy[-1], zz.max()),
                                         initialize=False)
                        #...

                zmax = numpy.max( (zz_1.max() * self.factor_surface1, zz_2.max() * self.factor_surface2, zz.max() ))
                self.plot_surface1(vmax=zmax)
                self.plot_surface2(vmax=zmax)
                self.plot_data2D(zz,xx,yy,self.tab_surfaceR,vmax=zmax)


                if not (self.surface_file_name.endswith("hd5") or self.surface_file_name.endswith("hdf5") or self.surface_file_name.endswith("hdf")):
                    self.surface_file_name += ".hdf5"

                OU.write_surface_file(zz, xx, yy, self.surface_file_name)
                self.writeStdOut("File written to disk: %s \n" % self.surface_file_name, initialize=False)

                error_profile_x_dim = abs(xx[-1] - xx[0])
                error_profile_y_dim = abs(yy[-1] - yy[0])


                self.send("PreProcessor_Data", OasysPreProcessorData(error_profile_data=OasysErrorProfileData(surface_data=OasysSurfaceData(xx=xx,
                                                                                                                                            yy=yy,
                                                                                                                                            zz=zz,
                                                                                                                                            surface_data_file=self.surface_file_name),
                                                                                                              error_profile_x_dim=error_profile_x_dim,
                                                                                                              error_profile_y_dim=error_profile_y_dim)))
                self.send("Surface_Data", OasysSurfaceData(xx=xx,
                                                           yy=yy,
                                                           zz=zz,
                                                           surface_data_file=self.surface_file_name))

        except Exception as exception:
            QMessageBox.critical(self, "Error",
                                 exception.args[0],
                                 QMessageBox.Ok)

            if self.IS_DEVELOP: raise exception



    def selectSurfaceFile(self):
        self.le_surface_file_name.setText(oasysgui.selectFileFromDialog(self, self.surface_file_name, "Select Input File", file_extension_filter="HDF5 Files (*.hdf5)"))


if __name__ == "__main__":
    import sys

    def get_surface(nx=100, ny=200, Lx = 0.2, Ly = 2.0):
        xx1 = numpy.linspace(-0.5 * Lx, 0.5 * Lx, nx)
        yy1 = numpy.linspace(-0.5 * Ly, 0.5 * Ly, ny)
        XX1 = numpy.outer(xx1,numpy.ones_like(yy1))
        YY1 = numpy.outer(numpy.ones_like(xx1), yy1)
        ZZ1 = numpy.exp( - XX1 ** 2 / (2 * 0.01**2) ) * \
              numpy.exp(- YY1 ** 2 / (2 * 0.5 ** 2))
        zz1 = ZZ1.T

        error_profile_x_dim = abs(xx1[-1] - xx1[0])
        error_profile_y_dim = abs(yy1[-1] - yy1[0])

        input1 = OasysPreProcessorData(error_profile_data=OasysErrorProfileData(surface_data=OasysSurfaceData(xx=xx1,
                                                                                                              yy=yy1,
                                                                                                              zz=zz1,
                                                                                                              surface_data_file=""),
                                                                                error_profile_x_dim=error_profile_x_dim,
                                                                                error_profile_y_dim=error_profile_y_dim))
        return input1

    input1 = get_surface(Ly=2.0,Lx=0.2,ny=200,nx=100)
    input2 = get_surface(Ly=2.0,Lx=0.2,ny=200,nx=100)
    # input2 = get_surface(Ly=1.0,Lx=0.1,ny=100,nx=50)



    a = QApplication(sys.argv)
    ow = OWSurfaceFileReader()

    ow.set_input_1(input1)
    ow.set_input_2(input2)
    # ow.compute()


    ow.show()

    a.exec_()
    ow.saveSettings()
